@@ -3,8 +3,8 @@
 
 #include <rz_analysis.h>
 #include <rz_lib.h>
-#include <capstone.h>
-#include <ppc.h>
+#include <capstone/capstone.h>
+#include <capstone/ppc.h>
 #include "../../asm/arch/ppc/libvle/vle.h"
 
 #define SPR_HID0 0x3f0 /* Hardware Implementation Register 0 */
@@ -81,7 +81,7 @@ static char *getarg2(struct Getarg *gop, int n, const char *setstr) {
 	switch (op.type) {
 	case PPC_OP_INVALID:
 		words[n][0] = '\0';
-		//strcpy (words[n], "invalid");
+		// strcpy (words[n], "invalid");
 		break;
 	case PPC_OP_REG:
 		snprintf(words[n], sizeof(words[n]),
@@ -207,7 +207,7 @@ static void opex(RzStrBuf *buf, csh handle, cs_insn *insn) {
 #define ARG(n)     getarg2(&gop, n, "")
 #define ARG2(n, m) getarg2(&gop, n, m)
 
-static bool set_reg_profile(RzAnalysis *analysis) {
+static char *get_reg_profile(RzAnalysis *analysis) {
 	const char *p = NULL;
 	if (analysis->bits == 32) {
 		p =
@@ -409,20 +409,23 @@ static bool set_reg_profile(RzAnalysis *analysis) {
 			"gpr	dbat1u .32 468 0\n"
 			"gpr	dbat2u .32 476 0\n"
 			"gpr	dbat3u .32 484 0\n"
-			"gpr	mask   .64 488 0\n"; //not a real register used on complex functions
+			"gpr	mask   .64 488 0\n"; // not a real register used on complex functions
 	}
-	return rz_reg_set_profile_string(analysis->reg, p);
+	return strdup(p);
 }
 
-static int analop_vle(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len) {
+static int analop_vle(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len, RzAnalysisOpMask mask) {
 	vle_t *instr = NULL;
 	vle_handle handle = { 0 };
 	op->size = 2;
 	if (len > 1 && !vle_init(&handle, buf, len) && (instr = vle_next(&handle))) {
 		op->size = instr->size;
 		op->type = instr->analysis_op;
-		//op->id = instr->type;
+		// op->id = instr->type;
 
+		if (mask & RZ_ANALYSIS_OP_MASK_DISASM) {
+			op->mnemonic = strdup(instr->name);
+		}
 		switch (op->type) {
 		case RZ_ANALYSIS_OP_TYPE_ILL:
 			break;
@@ -440,7 +443,7 @@ static int analop_vle(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf
 			op->fail = addr + op->size;
 			break;
 		case RZ_ANALYSIS_OP_TYPE_CJMP:
-			op->cond = instr->cond; //RZ_TYPE_COND_NE;
+			op->cond = instr->cond; // RZ_TYPE_COND_NE;
 			op->eob = true;
 			op->jump = addr + instr->fields[instr->n - 1].value;
 			op->fail = addr + op->size;
@@ -489,7 +492,7 @@ static int analop_vle(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf
 		case RZ_ANALYSIS_OP_TYPE_XOR:
 			break;
 		default:
-			//eprintf ("Missing an RZ_ANALYSIS_OP_TYPE (%"PFMT64u")\n", op->type);
+			// RZ_LOG_ERROR("Missing an RZ_ANALYSIS_OP_TYPE (%"PFMT64u")\n", op->type);
 			break;
 		}
 		vle_free(instr);
@@ -610,7 +613,7 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 		if (!a->big_endian) {
 			return -1;
 		}
-		ret = analop_vle(a, op, addr, buf, len);
+		ret = analop_vle(a, op, addr, buf, len, mask);
 		if (ret >= 0) {
 			return op->size;
 		}
@@ -636,6 +639,9 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 	if (n < 1) {
 		op->type = RZ_ANALYSIS_OP_TYPE_ILL;
 	} else {
+		if (mask & RZ_ANALYSIS_OP_MASK_DISASM) {
+			op->mnemonic = strdup(insn->mnemonic);
+		}
 		if (mask & RZ_ANALYSIS_OP_MASK_OPEX) {
 			opex(&op->opex, handle, insn);
 		}
@@ -681,10 +687,13 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 		case PPC_INS_MR:
 		case PPC_INS_LI:
 			op->type = RZ_ANALYSIS_OP_TYPE_MOV;
+			op->val = IMM(1);
 			esilprintf(op, "%s,%s,=", ARG(1), ARG(0));
 			break;
 		case PPC_INS_LIS:
 			op->type = RZ_ANALYSIS_OP_TYPE_MOV;
+			op->val = IMM(1);
+			op->val <<= 16;
 			esilprintf(op, "%s0000,%s,=", ARG(1), ARG(0));
 			break;
 		case PPC_INS_CLRLWI:
@@ -896,6 +905,7 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 			break;
 		case PPC_INS_MULLI:
 			op->sign = true;
+			// fallthrough
 		case PPC_INS_MULLW:
 		case PPC_INS_MULLD:
 			op->type = RZ_ANALYSIS_OP_TYPE_MUL;
@@ -909,8 +919,10 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 			op->type = RZ_ANALYSIS_OP_TYPE_SUB;
 			esilprintf(op, "%s,%s,-,%s,=", ARG(1), ARG(2), ARG(0));
 			break;
-		case PPC_INS_ADD:
 		case PPC_INS_ADDI:
+			op->val = ((st16)IMM(2));
+			// fallthrough
+		case PPC_INS_ADD:
 			op->sign = true;
 			op->type = RZ_ANALYSIS_OP_TYPE_ADD;
 			esilprintf(op, "%s,%s,+,%s,=", ARG(2), ARG(1), ARG(0));
@@ -1023,7 +1035,7 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 				}
 				op->jump = IMM(1);
 				op->fail = addr + op->size;
-				//op->type = RZ_ANALYSIS_OP_TYPE_UJMP;
+				// op->type = RZ_ANALYSIS_OP_TYPE_UJMP;
 			default:
 				break;
 			}
@@ -1283,7 +1295,7 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 			rz_strbuf_fini(&op->esil);
 		}
 		cs_free(insn, n);
-		//cs_close (&handle);
+		// cs_close (&handle);
 	}
 	return op->size;
 }
@@ -1312,7 +1324,7 @@ RzAnalysisPlugin rz_analysis_plugin_ppc_cs = {
 	.archinfo = archinfo,
 	.preludes = analysis_preludes,
 	.op = &analop,
-	.set_reg_profile = &set_reg_profile,
+	.get_reg_profile = &get_reg_profile,
 };
 
 #ifndef RZ_PLUGIN_INCORE
