@@ -2,13 +2,14 @@
 // SPDX-FileCopyrightText: 2009-2021 nibble <nibble.ds@gmail.com>
 // SPDX-License-Identifier: LGPL-3.0-only
 
+#include <rz_list.h>
 #include <stdio.h>
 #include <rz_core.h>
 #include <rz_types.h>
 #include <rz_util.h>
 #include <rz_asm.h>
 #define USE_R2 1
-#include <spp/spp.h>
+#include <spp.h>
 #include <config.h>
 
 RZ_LIB_VERSION(rz_asm);
@@ -55,7 +56,7 @@ static int rz_asm_pseudo_string(RzAsmOp *op, char *input, int zero) {
 
 static inline int rz_asm_pseudo_arch(RzAsm *a, char *input) {
 	if (!rz_asm_use(a, input)) {
-		eprintf("Error: Unknown plugin\n");
+		RZ_LOG_ERROR("Unknown asm plugin name '%s'\n", input);
 		return -1;
 	}
 	return 0;
@@ -63,7 +64,7 @@ static inline int rz_asm_pseudo_arch(RzAsm *a, char *input) {
 
 static inline int rz_asm_pseudo_bits(RzAsm *a, char *input) {
 	if (!(rz_asm_set_bits(a, rz_num_math(NULL, input)))) {
-		eprintf("Error: Unsupported value for .bits.\n");
+		RZ_LOG_ERROR("Unsupported bits (%s) value for the selected asm plugin.\n", input);
 		return -1;
 	}
 	return 0;
@@ -75,12 +76,11 @@ static inline int rz_asm_pseudo_org(RzAsm *a, char *input) {
 }
 
 static inline int rz_asm_pseudo_intN(RzAsm *a, RzAsmOp *op, char *input, int n) {
-	short s;
-	int i;
-	long int l;
+	ut16 s;
+	ut32 i;
 	ut64 s64 = rz_num_math(NULL, input);
 	if (n != 8 && s64 >> (n * 8)) {
-		eprintf("int16 Out is out of range\n");
+		RZ_LOG_ERROR("Cannot write a number that does not fit within a int%d type.\n", (n * 8));
 		return 0;
 	}
 	// XXX honor endian here
@@ -89,14 +89,13 @@ static inline int rz_asm_pseudo_intN(RzAsm *a, RzAsmOp *op, char *input, int n)
 		return 0;
 	}
 	if (n == 2) {
-		s = (short)s64;
+		s = (ut16)(st16)s64;
 		rz_write_ble16(buf, s, a->big_endian);
 	} else if (n == 4) {
-		i = (int)s64;
+		i = (ut32)(st32)s64;
 		rz_write_ble32(buf, i, a->big_endian);
 	} else if (n == 8) {
-		l = (long int)s64;
-		rz_write_ble64(buf, l, a->big_endian);
+		rz_write_ble64(buf, (ut64)s64, a->big_endian);
 	} else {
 		return 0;
 	}
@@ -158,12 +157,12 @@ static inline int rz_asm_pseudo_incbin(RzAsmOp *op, char *input) {
 	rz_str_replace_char(input, ',', ' ');
 	// int len = rz_str_word_count (input);
 	rz_str_word_set0(input);
-	//const char *filename = rz_str_word_get0 (input, 0);
+	// const char *filename = rz_str_word_get0 (input, 0);
 	size_t skip = (size_t)rz_num_math(NULL, rz_str_word_get0(input, 1));
 	size_t count = (size_t)rz_num_math(NULL, rz_str_word_get0(input, 2));
 	char *content = rz_file_slurp(input, &bytes_read);
 	if (!content) {
-		eprintf("Could not open '%s'.\n", input);
+		RZ_LOG_ERROR("Could not open '%s'.\n", input);
 		return -1;
 	}
 	if (skip > 0) {
@@ -252,6 +251,7 @@ RZ_API void rz_asm_free(RzAsm *a) {
 	}
 	rz_syscall_free(a->syscall);
 	free(a->cpu);
+	free(a->features);
 	sdb_free(a->pair);
 	ht_pp_free(a->flags);
 	a->pair = NULL;
@@ -305,6 +305,46 @@ RZ_API bool rz_asm_use_assembler(RzAsm *a, const char *name) {
 	return false;
 }
 
+/**
+ * \brief Copies all config nodes in \p pcfg to the config in \p rz_asm.
+ *
+ * \param rz_asm Pointer to RzAsm struct.
+ * \param pcfg Pointer to the plugins RzConfig struct.
+ */
+static void set_plugin_configs(RZ_BORROW RzAsm *rz_asm, RZ_BORROW RzConfig *pcfg) {
+	rz_return_if_fail(pcfg && rz_asm);
+
+	RzConfig *conf = ((RzCore *)(rz_asm->core))->config;
+	RzConfigNode *n;
+	RzListIter *it;
+	rz_list_foreach_iter(pcfg->nodes, it) {
+		n = it->data;
+		if (!rz_config_add_node(conf, rz_config_node_clone(n))) {
+			RZ_LOG_WARN("Failed to add \"%s\" to the global config.\n", n->name)
+		}
+	}
+}
+
+/**
+ * \brief Deletes all copies of \p pcfg nodes in the RzConfig from \p rz_asm.
+ *
+ * \param rz_asm Pointer to RzAsm struct.
+ * \param pcfg Pointer to the plugins RzConfig struct.
+ */
+static void unset_plugins_config(RZ_BORROW RzAsm *rz_asm, RZ_BORROW RzConfig *pcfg) {
+	rz_return_if_fail(pcfg && rz_asm && rz_asm->core);
+
+	RzConfig *conf = ((RzCore *)(rz_asm->core))->config;
+	RzConfigNode *n;
+	RzListIter *it;
+	rz_list_foreach_iter(pcfg->nodes, it) {
+		n = it->data;
+		if (!rz_config_rm(conf, n->name)) {
+			RZ_LOG_WARN("Failed to remove \"%s\" from the global config.", n->name)
+		}
+	}
+}
+
 // TODO: this can be optimized using rz_str_hash()
 RZ_API bool rz_asm_use(RzAsm *a, const char *name) {
 	RzAsmPlugin *h;
@@ -312,6 +352,7 @@ RZ_API bool rz_asm_use(RzAsm *a, const char *name) {
 	if (!a || !name) {
 		return false;
 	}
+	RzCore *core = a->core;
 	if (a->cur && !strcmp(a->cur->arch, name)) {
 		return true;
 	}
@@ -319,20 +360,30 @@ RZ_API bool rz_asm_use(RzAsm *a, const char *name) {
 		if (h->arch && h->name && !strcmp(h->name, name)) {
 			if (!a->cur || (a->cur && strcmp(a->cur->arch, h->arch))) {
 				plugin_fini(a);
-				char *rzprefix = rz_str_rz_prefix(RZ_SDB_OPCODES);
-				char *file = rz_str_newf("%s/%s.sdb", rz_str_get_null(rzprefix), h->arch);
+				char *opcodes_dir = rz_path_system(RZ_SDB_OPCODES);
+				char *file = rz_str_newf("%s/%s.sdb", opcodes_dir, h->arch);
 				if (file) {
 					rz_asm_set_cpu(a, NULL);
 					sdb_free(a->pair);
 					a->pair = sdb_new(NULL, file, 0);
 					free(file);
 				}
-				free(rzprefix);
+				free(opcodes_dir);
 			}
 			if (h->init && !h->init(&a->plugin_data)) {
 				RZ_LOG_ERROR("asm plugin '%s' failed to initialize.\n", h->name);
 				return false;
 			}
+			if (a->cur && a->cur->get_config && core) {
+				rz_config_lock(core->config, false);
+				unset_plugins_config(a, a->cur->get_config());
+				rz_config_lock(core->config, true);
+			}
+			if (h->get_config && core) {
+				rz_config_lock(core->config, false);
+				set_plugin_configs(a, h->get_config());
+				rz_config_lock(core->config, true);
+			}
 			a->cur = h;
 			return true;
 		}
@@ -377,7 +428,7 @@ RZ_API bool rz_asm_set_big_endian(RzAsm *a, bool b) {
 		a->big_endian = true;
 		break;
 	default:
-		eprintf("RzAsmPlugin doesn't specify endianness\n");
+		RZ_LOG_DEBUG("The asm plugin doesn't specify endianness.\n");
 		break;
 	}
 	return a->big_endian;
@@ -614,7 +665,6 @@ RZ_API RzAsmCode *rz_asm_mdisassemble(RzAsm *a, const ut8 *buf, int len) {
 	RzStrBuf *buf_asm;
 	RzAsmCode *acode;
 	ut64 pc = a->pc;
-	RzAsmOp op;
 	ut64 idx;
 	size_t ret;
 	const size_t addrbytes = a->core ? ((RzCore *)a->core)->io->addrbytes : 1;
@@ -629,6 +679,8 @@ RZ_API RzAsmCode *rz_asm_mdisassemble(RzAsm *a, const ut8 *buf, int len) {
 	if (!(buf_asm = rz_strbuf_new(NULL))) {
 		return rz_asm_code_free(acode);
 	}
+	RzAsmOp op;
+	rz_asm_op_init(&op);
 	for (idx = 0; idx + addrbytes <= len; idx += (addrbytes * ret)) {
 		rz_asm_set_pc(a, pc + idx);
 		ret = rz_asm_disassemble(a, &op, buf + idx, len - idx);
@@ -641,6 +693,7 @@ RZ_API RzAsmCode *rz_asm_mdisassemble(RzAsm *a, const ut8 *buf, int len) {
 		rz_strbuf_append(buf_asm, rz_strbuf_get(&op.buf_asm));
 		rz_strbuf_append(buf_asm, "\n");
 	}
+	rz_asm_op_fini(&op);
 	acode->assembly = rz_strbuf_drain(buf_asm);
 	acode->len = idx;
 	return acode;
@@ -763,12 +816,12 @@ RZ_API RzAsmCode *rz_asm_massemble(RzAsm *a, const char *assembly) {
 			const size_t new_tokens_size = tokens_size * 2;
 			if (sizeof(char *) * new_tokens_size <= sizeof(char *) * tokens_size) {
 				// overflow
-				eprintf("Too many tokens\n");
+				RZ_LOG_ERROR("Too many tokens while assembling (overflow).\n");
 				goto fail;
 			}
 			char **new_tokens = realloc(tokens, sizeof(char *) * new_tokens_size);
 			if (!new_tokens) {
-				eprintf("Too many tokens\n");
+				RZ_LOG_ERROR("Cannot reallocate meory for tokens while assembling.\n");
 				goto fail;
 			}
 			tokens_size = new_tokens_size;
@@ -861,7 +914,7 @@ RZ_API RzAsmCode *rz_asm_massemble(RzAsm *a, const char *assembly) {
 					q++;
 				}
 				if (is_a_label) {
-					//if (stage != 2) {
+					// if (stage != 2) {
 					if (ptr_start[1] && ptr_start[1] != ' ') {
 						*ptr = 0;
 						char *p = strdup(ptr_start);
@@ -944,7 +997,7 @@ RZ_API RzAsmCode *rz_asm_massemble(RzAsm *a, const char *assembly) {
 				} else if ((!strncmp(ptr, ".byte ", 6)) || (!strncmp(ptr, ".int8 ", 6))) {
 					ret = rz_asm_pseudo_byte(&op, ptr + 6);
 				} else if (!strncmp(ptr, ".glob", 5)) { // .global .globl
-					//	eprintf (".global directive not yet implemented\n");
+					RZ_LOG_DEBUG(".global directive not yet implemented\n");
 					ret = 0;
 					continue;
 				} else if (!strncmp(ptr, ".equ ", 5)) {
@@ -959,31 +1012,31 @@ RZ_API RzAsmCode *rz_asm_massemble(RzAsm *a, const char *assembly) {
 						*ptr2 = '\0';
 						rz_asm_code_set_equ(acode, ptr + 5, ptr2 + 1);
 					} else {
-						eprintf("Invalid syntax for '.equ': Use '.equ <word> <word>'\n");
+						RZ_LOG_ERROR("Invalid syntax for '.equ': Use '.equ <word> <word>'\n");
 					}
 				} else if (!strncmp(ptr, ".org ", 5)) {
 					ret = rz_asm_pseudo_org(a, ptr + 5);
 				} else if (rz_str_startswith(ptr, ".offset ")) {
-					eprintf("Invalid use of the .offset directory. This directive is only supported in rizin -c 'waf'.\n");
+					RZ_LOG_ERROR("Invalid use of the .offset directory. This directive is only supported in rizin -c 'waf'.\n");
 				} else if (!strncmp(ptr, ".text", 5)) {
 					acode->code_offset = a->pc;
 				} else if (!strncmp(ptr, ".data", 5)) {
 					acode->data_offset = a->pc;
 				} else if (!strncmp(ptr, ".incbin", 7)) {
 					if (ptr[7] != ' ') {
-						eprintf("incbin missing filename\n");
+						RZ_LOG_ERROR("Invalid syntax for '.incbin': Use '.incbin <filename>'\n");
 						continue;
 					}
 					ret = rz_asm_pseudo_incbin(&op, ptr + 8);
 				} else {
-					eprintf("Unknown directive (%s)\n", ptr);
+					RZ_LOG_ERROR("Unknown directive named '%s'\n", ptr);
 					goto fail;
 				}
 				if (!ret) {
 					continue;
 				}
 				if (ret < 0) {
-					eprintf("!!! Oops (%s)\n", ptr);
+					RZ_LOG_ERROR("Something went wrong when handling the directive '%s'.\n", ptr);
 					goto fail;
 				}
 			} else { /* Instruction */
@@ -994,18 +1047,22 @@ RZ_API RzAsmCode *rz_asm_massemble(RzAsm *a, const char *assembly) {
 						continue;
 					}
 					str = rz_asm_code_equ_replace(acode, strdup(ptr_start));
+					rz_asm_op_fini(&op);
+					rz_asm_op_init(&op);
 					ret = rz_asm_assemble(a, &op, str);
 					free(str);
 				} else {
 					if (!*ptr_start) {
 						continue;
 					}
+					rz_asm_op_fini(&op);
+					rz_asm_op_init(&op);
 					ret = rz_asm_assemble(a, &op, ptr_start);
 				}
 			}
 			if (stage == STAGES - 1) {
 				if (ret < 1) {
-					eprintf("Cannot assemble '%s' at line %d\n", ptr_start, linenum);
+					RZ_LOG_ERROR("Cannot assemble '%s' at line %d\n", ptr_start, linenum);
 					goto fail;
 				}
 				acode->len = idx + ret;
@@ -1028,19 +1085,17 @@ RZ_API RzAsmCode *rz_asm_massemble(RzAsm *a, const char *assembly) {
 			}
 		}
 	}
+	rz_asm_op_fini(&op);
 	free(lbuf);
 	free(tokens);
 	return acode;
 fail:
+	rz_asm_op_fini(&op);
 	free(lbuf);
 	free(tokens);
 	return rz_asm_code_free(acode);
 }
 
-RZ_API bool rz_asm_modify(RzAsm *a, ut8 *buf, int field, ut64 val) {
-	return (a->cur && a->cur->modify) ? a->cur->modify(a, buf, field, val) : false;
-}
-
 RZ_API int rz_asm_get_offset(RzAsm *a, int type, int idx) { // link to rbin
 	if (a && a->binb.bin && a->binb.get_offset) {
 		return a->binb.get_offset(a->binb.bin, type, idx);
