@@ -4,7 +4,7 @@
 // SPDX-License-Identifier: LGPL-3.0-only
 
 #include <rz_bin.h>
-#include <rz_types.h>
+#include <rz_demangler.h>
 #include <rz_util.h>
 #include <rz_lib.h>
 #include <rz_io.h>
@@ -71,13 +71,19 @@ RZ_API RzBinXtrData *rz_bin_xtrdata_new(RzBuffer *buf, ut64 offset, ut64 size, u
 
 RZ_API RZ_BORROW const char *rz_bin_string_type(int type) {
 	switch (type) {
-	case RZ_STRING_TYPE_ASCII: return "ascii";
-	case RZ_STRING_TYPE_UTF8: return "utf8";
-	case RZ_STRING_TYPE_WIDE_LE: return "utf16le";
-	case RZ_STRING_TYPE_WIDE32_LE: return "utf32le";
-	case RZ_STRING_TYPE_WIDE_BE: return "utf16be";
-	case RZ_STRING_TYPE_WIDE32_BE: return "utf32be";
-	case RZ_STRING_TYPE_BASE64: return "base64";
+	case RZ_BIN_STRING_ENC_8BIT: return "ascii";
+	case RZ_BIN_STRING_ENC_UTF8: return "utf8";
+	case RZ_BIN_STRING_ENC_MUTF8: return "mutf8";
+	case RZ_BIN_STRING_ENC_WIDE_LE: return "utf16le";
+	case RZ_BIN_STRING_ENC_WIDE32_LE: return "utf32le";
+	case RZ_BIN_STRING_ENC_WIDE_BE: return "utf16be";
+	case RZ_BIN_STRING_ENC_WIDE32_BE: return "utf32be";
+	case RZ_BIN_STRING_ENC_BASE64: return "base64";
+	case RZ_STRING_ENC_IBM037: return "ibm037";
+	case RZ_STRING_ENC_IBM290: return "ibm290";
+	case RZ_STRING_ENC_EBCDIC_ES: return "ebcdices";
+	case RZ_STRING_ENC_EBCDIC_UK: return "ebcdicuk";
+	case RZ_STRING_ENC_EBCDIC_US: return "ebcdicus";
 	}
 	return "ascii"; // XXX
 }
@@ -96,24 +102,13 @@ RZ_API void rz_bin_xtrdata_free(void /*RzBinXtrData*/ *data_) {
 	free(data);
 }
 
-RZ_API RzList *rz_bin_raw_strings(RzBinFile *bf, int min) {
-	rz_return_val_if_fail(bf, NULL);
-	return rz_bin_file_get_strings(bf, min, 0, 2);
-}
-
-RZ_API RzList *rz_bin_dump_strings(RzBinFile *bf, int min, int raw) {
-	rz_return_val_if_fail(bf, NULL);
-	return rz_bin_file_get_strings(bf, min, 1, raw);
-}
-
-RZ_API void rz_bin_options_init(RzBinOptions *opt, int fd, ut64 baseaddr, ut64 loadaddr, bool patch_relocs, int rawstr) {
+RZ_API void rz_bin_options_init(RzBinOptions *opt, int fd, ut64 baseaddr, ut64 loadaddr, bool patch_relocs) {
 	memset(opt, 0, sizeof(*opt));
 	opt->obj_opts.baseaddr = baseaddr;
 	opt->obj_opts.loadaddr = loadaddr;
 	opt->obj_opts.patch_relocs = patch_relocs;
 	opt->obj_opts.elf_load_sections = true;
 	opt->fd = fd;
-	opt->rawstr = rawstr;
 }
 
 RZ_API void rz_bin_arch_options_init(RzBinArchOptions *opt, const char *arch, int bits) {
@@ -144,6 +139,7 @@ RZ_API void rz_bin_info_free(RzBinInfo *rb) {
 	free(rb->cpu);
 	free(rb->machine);
 	free(rb->os);
+	free(rb->features);
 	free(rb->subsystem);
 	free(rb->default_cc);
 	free(rb->rpath);
@@ -178,6 +174,17 @@ RZ_API void rz_bin_import_free(RzBinImport *imp) {
 	}
 }
 
+RZ_API void rz_bin_resource_free(RzBinResource *res) {
+	if (!res) {
+		return;
+	}
+	RZ_FREE(res->name);
+	RZ_FREE(res->time);
+	RZ_FREE(res->type);
+	RZ_FREE(res->language);
+	free(res);
+}
+
 RZ_API const char *rz_bin_symbol_name(RzBinSymbol *s) {
 	if (s->dup_count) {
 		return sdb_fmt("%s_%d", s->name, s->dup_count);
@@ -228,7 +235,7 @@ RZ_API RzBinFile *rz_bin_open(RzBin *bin, const char *file, RzBinOptions *opt) {
 		opt->fd = iob->fd_open(iob->io, file, RZ_PERM_R, 0644);
 	}
 	if (opt->fd < 0) {
-		eprintf("Couldn't open bin for file '%s'\n", file);
+		RZ_LOG_ERROR("Couldn't open bin for file '%s'\n", file);
 		return NULL;
 	}
 	opt->sz = 0;
@@ -239,16 +246,18 @@ RZ_API RzBinFile *rz_bin_open(RzBin *bin, const char *file, RzBinOptions *opt) {
 RZ_API RzBinFile *rz_bin_reload(RzBin *bin, RzBinFile *bf, ut64 baseaddr) {
 	rz_return_val_if_fail(bin && bf, NULL);
 
+	bool big_endian = bf->o ? bf->o->opts.big_endian : false;
 	bool patch_relocs = bf->o ? bf->o->opts.patch_relocs : false;
 	bool elf_load_sections = bf->o ? bf->o->opts.elf_load_sections : false;
 	bool elf_checks_sections = bf->o ? bf->o->opts.elf_checks_sections : false;
 	bool elf_checks_segments = bf->o ? bf->o->opts.elf_checks_segments : false;
 
 	RzBinOptions opt;
-	rz_bin_options_init(&opt, bf->fd, baseaddr, bf->loadaddr, patch_relocs, bin->rawstr);
+	rz_bin_options_init(&opt, bf->fd, baseaddr, bf->loadaddr, patch_relocs);
 	opt.obj_opts.elf_load_sections = elf_load_sections;
 	opt.obj_opts.elf_checks_sections = elf_checks_sections;
 	opt.obj_opts.elf_checks_segments = elf_checks_segments;
+	opt.obj_opts.big_endian = big_endian;
 	opt.filename = bf->file;
 	rz_buf_seek(bf->buf, 0, RZ_BUF_SET);
 	RzBinFile *nbf = rz_bin_open_buf(bin, bf->buf, &opt);
@@ -262,7 +271,6 @@ RZ_API RzBinFile *rz_bin_open_buf(RzBin *bin, RzBuffer *buf, RzBinOptions *opt)
 	RzListIter *it;
 	RzBinXtrPlugin *xtr;
 
-	bin->rawstr = opt->rawstr;
 	bin->file = opt->filename;
 	if (opt->obj_opts.loadaddr == UT64_MAX) {
 		opt->obj_opts.loadaddr = 0;
@@ -275,7 +283,7 @@ RZ_API RzBinFile *rz_bin_open_buf(RzBin *bin, RzBuffer *buf, RzBinOptions *opt)
 		// <xtr_name>:<bin_type_name>
 		rz_list_foreach (bin->binxtrs, it, xtr) {
 			if (!xtr->check_buffer) {
-				eprintf("Missing check_buffer callback for '%s'\n", xtr->name);
+				RZ_LOG_ERROR("Missing check_buffer callback for '%s'\n", xtr->name);
 				continue;
 			}
 			if (xtr->check_buffer(buf)) {
@@ -283,7 +291,7 @@ RZ_API RzBinFile *rz_bin_open_buf(RzBin *bin, RzBuffer *buf, RzBinOptions *opt)
 					xtr->extract_from_bytes || xtr->extractall_from_bytes) {
 					bf = rz_bin_file_xtr_load_buffer(bin, xtr,
 						bin->file, buf, &opt->obj_opts,
-						opt->xtr_idx, opt->fd, bin->rawstr);
+						opt->xtr_idx, opt->fd);
 				}
 			}
 		}
@@ -291,7 +299,7 @@ RZ_API RzBinFile *rz_bin_open_buf(RzBin *bin, RzBuffer *buf, RzBinOptions *opt)
 	if (!bf) {
 		// Uncomment for this speedup: 20s vs 22s
 		// RzBuffer *buf = rz_buf_new_slurp (bin->file);
-		bf = rz_bin_file_new_from_buffer(bin, bin->file, buf, bin->rawstr,
+		bf = rz_bin_file_new_from_buffer(bin, bin->file, buf,
 			&opt->obj_opts, opt->fd, opt->pluginname);
 		if (!bf) {
 			return NULL;
@@ -326,7 +334,7 @@ RZ_API RzBinFile *rz_bin_open_io(RzBin *bin, RzBinOptions *opt) {
 		is_debugger = false;
 	}
 	if (!buf) {
-		buf = rz_buf_new_with_io(&bin->iob, opt->fd);
+		buf = rz_buf_new_with_io_fd(&bin->iob, opt->fd);
 	}
 	if (!buf) {
 		return NULL;
@@ -384,6 +392,24 @@ RZ_API RzBinPlugin *rz_bin_get_binplugin_by_buffer(RzBin *bin, RzBuffer *buf) {
 	return NULL;
 }
 
+RZ_IPI RzBinPlugin *rz_bin_get_binplugin_by_filename(RzBin *bin) {
+	RzBinPlugin *plugin;
+	RzListIter *it;
+
+	rz_return_val_if_fail(bin, NULL);
+
+	const char *filename = strrchr(bin->file, RZ_SYS_DIR[0]);
+	filename = filename ? filename + 1 : bin->file;
+	rz_list_foreach (bin->plugins, it, plugin) {
+		if (plugin->check_filename) {
+			if (plugin->check_filename(filename)) {
+				return plugin;
+			}
+		}
+	}
+	return NULL;
+}
+
 RZ_IPI RzBinXtrPlugin *rz_bin_get_xtrplugin_by_name(RzBin *bin, const char *name) {
 	RzBinXtrPlugin *xtr;
 	RzListIter *it;
@@ -468,7 +494,7 @@ RZ_API void rz_bin_free(RzBin *bin) {
 	free(bin->force);
 	free(bin->srcdir);
 	free(bin->strenc);
-	//rz_bin_free_bin_files (bin);
+	// rz_bin_free_bin_files (bin);
 	rz_list_free(bin->binfiles);
 	rz_list_free(bin->binxtrs);
 	rz_list_free(bin->plugins);
@@ -477,6 +503,7 @@ RZ_API void rz_bin_free(RzBin *bin) {
 	rz_id_storage_free(bin->ids);
 	rz_event_free(bin->event);
 	rz_str_constpool_fini(&bin->constpool);
+	rz_demangler_free(bin->demangler);
 	free(bin);
 }
 
@@ -550,7 +577,7 @@ RZ_API bool rz_bin_list_plugin(RzBin *bin, const char *name, PJ *pj, int json) {
 		return true;
 	}
 
-	eprintf("Cannot find plugin %s\n", name);
+	RZ_LOG_ERROR("Cannot find plugin %s\n", name);
 	return false;
 }
 
@@ -572,25 +599,17 @@ RZ_API void rz_bin_set_baddr(RzBin *bin, ut64 baddr) {
 	rz_return_if_fail(bin);
 	RzBinFile *bf = rz_bin_cur(bin);
 	RzBinObject *o = rz_bin_cur_object(bin);
-	if (o) {
-		if (!o->plugin || !o->plugin->baddr) {
-			return;
-		}
-		ut64 file_baddr = o->plugin->baddr(bf);
-		if (baddr == UT64_MAX) {
-			o->opts.baseaddr = file_baddr;
-			o->baddr_shift = 0; // o->baddr; // - file_baddr;
-		} else {
-			if (file_baddr != UT64_MAX) {
-				o->opts.baseaddr = baddr;
-				o->baddr_shift = baddr - file_baddr;
-			}
-		}
-	} else {
-		eprintf("Warning: This should be an assert probably.\n");
+	if (!o || !o->plugin || !o->plugin->baddr) {
+		return;
+	}
+	ut64 file_baddr = o->plugin->baddr(bf);
+	if (baddr == UT64_MAX) {
+		o->opts.baseaddr = file_baddr;
+		o->baddr_shift = 0; // o->baddr; // - file_baddr;
+	} else if (file_baddr != UT64_MAX) {
+		o->opts.baseaddr = baddr;
+		o->baddr_shift = baddr - file_baddr;
 	}
-	// XXX - update all the infos?
-	// maybe in RzBinFile.rebase() ?
 }
 
 // XXX: those accessors are redundant
@@ -630,6 +649,14 @@ RZ_API RzList *rz_bin_get_sections(RzBin *bin) {
 	return o ? (RzList *)rz_bin_object_get_sections_all(o) : NULL;
 }
 
+/**
+ * \brief Find the binary section at offset \p off.
+ *
+ * \param o Reference to the \p RzBinObject instance
+ * \param off Address to search
+ * \param va When 0 the offset \p off is considered a physical address, otherwise a virtual address
+ * \return Pointer to a \p RzBinSection containing the address
+ */
 RZ_API RzBinSection *rz_bin_get_section_at(RzBinObject *o, ut64 off, int va) {
 	RzBinSection *section;
 	RzListIter *iter;
@@ -650,6 +677,65 @@ RZ_API RzBinSection *rz_bin_get_section_at(RzBinObject *o, ut64 off, int va) {
 	return NULL;
 }
 
+/**
+ * \brief Find the last binary map at offset \p off .
+ *
+ * This function returns the last binary map that contains offset \p off,
+ * because it assumes that maps are sorted by priority, thus the last one will
+ * be the most important one.
+ *
+ * \param o Reference to the \p RzBinObject instance
+ * \param off Address to search
+ * \param va When false the offset \p off is considered a physical address, otherwise a virtual address
+ * \return Pointer to a \p RzBinMap containing the address
+ */
+RZ_API RzBinMap *rz_bin_object_get_map_at(RzBinObject *o, ut64 off, bool va) {
+	rz_return_val_if_fail(o, NULL);
+
+	RzBinMap *map;
+	RzListIter *iter;
+	ut64 from, to;
+
+	rz_list_foreach_prev(o->maps, iter, map) {
+		from = va ? rz_bin_object_addr_with_base(o, map->vaddr) : map->paddr;
+		to = from + (va ? map->vsize : map->psize);
+		if (off >= from && off < to) {
+			return map;
+		}
+	}
+	return NULL;
+}
+
+/**
+ * \brief Find all binary maps at offset \p off .
+ *
+ * \param o Reference to the \p RzBinObject instance
+ * \param off Address to search
+ * \param va When false the offset \p off is considered a physical address, otherwise a virtual address
+ * \return Vector of \p RzBinMap pointers
+ */
+RZ_API RZ_OWN RzPVector *rz_bin_object_get_maps_at(RzBinObject *o, ut64 off, bool va) {
+	rz_return_val_if_fail(o, NULL);
+
+	RzBinMap *map;
+	RzListIter *iter;
+	ut64 from, to;
+
+	RzPVector *res = rz_pvector_new(NULL);
+	if (!res) {
+		return NULL;
+	}
+
+	rz_list_foreach (o->maps, iter, map) {
+		from = va ? rz_bin_object_addr_with_base(o, map->vaddr) : map->paddr;
+		to = from + (va ? map->vsize : map->psize);
+		if (off >= from && off < to) {
+			rz_pvector_push(res, map);
+		}
+	}
+	return res;
+}
+
 RZ_API RzList *rz_bin_reset_strings(RzBin *bin) {
 	rz_return_val_if_fail(bin, NULL);
 	RzBinFile *bf = rz_bin_cur(bin);
@@ -705,9 +791,14 @@ RZ_API RzBin *rz_bin_new(void) {
 	if (!bin) {
 		return NULL;
 	}
-	if (!rz_str_constpool_init(&bin->constpool)) {
+	/* demanglers */
+	bin->demangler = rz_demangler_new();
+	if (!bin->demangler) {
 		goto trashbin;
 	}
+	if (!rz_str_constpool_init(&bin->constpool)) {
+		goto trashbin_demangler;
+	}
 	bin->event = rz_event_new(bin);
 	if (!bin->event) {
 		goto trashbin_constpool;
@@ -755,7 +846,9 @@ RZ_API RzBin *rz_bin_new(void) {
 			free(static_ldr_plugin);
 		}
 	}
+
 	return bin;
+
 trashbin_binldrs:
 	rz_list_free(bin->binldrs);
 trashbin_binxtrs:
@@ -765,6 +858,8 @@ trashbin_binxtrs:
 	rz_event_free(bin->event);
 trashbin_constpool:
 	rz_str_constpool_fini(&bin->constpool);
+trashbin_demangler:
+	rz_demangler_free(bin->demangler);
 trashbin:
 	free(bin);
 	return NULL;
@@ -866,73 +961,6 @@ RZ_API RzBuffer *rz_bin_create(RzBin *bin, const char *p,
 	return plugin->create(bin, code, codelen, data, datalen, opt);
 }
 
-RZ_API RzBuffer *rz_bin_package(RzBin *bin, const char *type, const char *file, RzList *files) {
-	if (!strcmp(type, "zip")) {
-		// XXX: implement me
-		rz_warn_if_reached();
-	} else if (!strcmp(type, "fat")) {
-		// XXX: this should be implemented in the fat plugin, not here
-		// XXX should pick the callback from the plugin list
-		const char *f;
-		RzListIter *iter;
-		ut32 num;
-		ut8 *num8 = (ut8 *)&num;
-		RzBuffer *buf = rz_buf_new_file(file, O_RDWR | O_CREAT, 0644);
-		if (!buf) {
-			eprintf("Cannot open file %s - Permission Denied.\n", file);
-			return NULL;
-		}
-		rz_buf_write_at(buf, 0, (const ut8 *)"\xca\xfe\xba\xbe", 4);
-		int count = rz_list_length(files);
-
-		num = rz_read_be32(&count);
-		ut64 from = 0x1000;
-		rz_buf_write_at(buf, 4, num8, 4);
-		int off = 12;
-		int item = 0;
-		rz_list_foreach (files, iter, f) {
-			size_t f_len = 0;
-			ut8 *f_buf = (ut8 *)rz_file_slurp(f, &f_len);
-			if (f_buf) {
-				eprintf("ADD %s %" PFMT64u "\n", f, (ut64)f_len);
-			} else {
-				eprintf("Cannot open %s\n", f);
-				free(f_buf);
-				continue;
-			}
-			item++;
-			/* CPU */
-			num8[0] = f_buf[7];
-			num8[1] = f_buf[6];
-			num8[2] = f_buf[5];
-			num8[3] = f_buf[4];
-			rz_buf_write_at(buf, off - 4, num8, 4);
-			/* SUBTYPE */
-			num8[0] = f_buf[11];
-			num8[1] = f_buf[10];
-			num8[2] = f_buf[9];
-			num8[3] = f_buf[8];
-			rz_buf_write_at(buf, off, num8, 4);
-			ut32 from32 = from;
-			/* FROM */
-			num = rz_read_be32(&from32);
-			rz_buf_write_at(buf, off + 4, num8, 4);
-			rz_buf_write_at(buf, from, f_buf, f_len);
-			/* SIZE */
-			num = rz_read_be32(&f_len);
-			rz_buf_write_at(buf, off + 8, num8, 4);
-			off += 20;
-			from += f_len + (f_len % 0x1000);
-			free(f_buf);
-		}
-		rz_buf_free(buf);
-		return NULL;
-	} else {
-		eprintf("Usage: rz-bin -X [fat|zip] [filename] [files ...]\n");
-	}
-	return NULL;
-}
-
 RZ_API RzList * /*<RzBinClass>*/ rz_bin_get_classes(RzBin *bin) {
 	rz_return_val_if_fail(bin, NULL);
 	RzBinObject *o = rz_bin_cur_object(bin);
@@ -1211,16 +1239,15 @@ RZ_API RZ_OWN RzList *rz_bin_section_flag_to_list(RzBin *bin, ut64 flag) {
 }
 
 RZ_API RzBinFile *rz_bin_file_at(RzBin *bin, ut64 at) {
-	RzListIter *it, *it2;
+	RzListIter *it;
 	RzBinFile *bf;
-	RzBinSection *s;
 	rz_list_foreach (bin->binfiles, it, bf) {
-		// chk for baddr + size of no section is covering anything
-		// we should honor maps not sections imho
-		rz_list_foreach (bf->o->sections, it2, s) {
-			if (at >= s->vaddr && at < (s->vaddr + s->vsize)) {
-				return bf;
-			}
+		if (!bf->o) {
+			continue;
+		}
+		RzBinMap *map = rz_bin_object_get_map_at(bf->o, at, true);
+		if (map) {
+			return bf;
 		}
 		if (at >= bf->o->opts.baseaddr && at < (bf->o->opts.baseaddr + bf->size)) {
 			return bf;
@@ -1295,3 +1322,177 @@ RZ_API const RzBinLdrPlugin *rz_bin_ldrplugin_get(RzBin *bin, const char *name)
 	}
 	return NULL;
 }
+
+#if WITH_GPL
+static char *bin_demangle_cxx(RzBinFile *bf, const char *symbol, ut64 vaddr) {
+	char *out = rz_demangler_cxx(symbol);
+	if (!out || !bf) {
+		return out;
+	}
+	char *sign = (char *)strchr(out, '(');
+	if (!sign) {
+		return out;
+	}
+
+	char *str = out;
+	char *ptr = NULL;
+	char *method_name = NULL;
+	for (;;) {
+		ptr = strstr(str, "::");
+		if (!ptr || ptr > sign) {
+			break;
+		}
+		method_name = ptr;
+		str = ptr + 1;
+	}
+
+	if (RZ_STR_ISEMPTY(method_name)) {
+		return out;
+	}
+
+	*method_name = 0;
+	RzBinSymbol *sym = rz_bin_file_add_method(bf, out, method_name + 2, 0);
+	if (sym) {
+		if (sym->vaddr != 0 && sym->vaddr != vaddr) {
+			RZ_LOG_INFO("Duplicated method found: %s\n", sym->name);
+		}
+		if (sym->vaddr == 0) {
+			sym->vaddr = vaddr;
+		}
+	}
+	*method_name = ':';
+	return out;
+}
+
+static char *bin_demangle_rust(RzBinFile *binfile, const char *symbol, ut64 vaddr) {
+	char *str = NULL;
+	if (!(str = bin_demangle_cxx(binfile, symbol, vaddr))) {
+		return str;
+	}
+	free(str);
+	return rz_demangler_rust(symbol);
+}
+#endif
+
+/**
+ * \brief Demangles a symbol based on the language or the RzBinFile data
+ *
+ * This function demangles a symbol based on the language or the RzBinFile data
+ * When C++ or rust is selected as the language, it will add methods into the
+ * RzBinFile structure based on the demangled symbol.
+ * When libs is set to true, the demangled symbol will be appended to the
+ * library name <libname>_<demangled symbol>.
+ *
+ * \param bf RzBinFile data to be used for demangling
+ * \param language Language to be used for demanglind
+ * \param symbol Symbol to be demangled
+ * \param vaddr vaddr of the \p symbol to be demangled
+ * \param libs Append the library name to the demangled symbol, if set to true
+ * \return char* Demangled name of the \p symbol
+ */
+RZ_API RZ_OWN char *rz_bin_demangle(RZ_NULLABLE RzBinFile *bf, RZ_NULLABLE const char *language, RZ_NULLABLE const char *symbol, ut64 vaddr, bool libs) {
+	if (RZ_STR_ISEMPTY(symbol)) {
+		return NULL;
+	}
+
+	RzBinLanguage type = RZ_BIN_LANGUAGE_UNKNOWN;
+	RzBin *bin = bf ? bf->rbin : NULL;
+	RzBinObject *o = bf ? bf->o : NULL;
+
+	if (!language && o && o->info && o->info->lang) {
+		language = o->info->lang;
+	}
+
+	RzListIter *iter;
+	const char *lib = NULL;
+	if (!strncmp(symbol, "reloc.", 6)) {
+		symbol += 6;
+	}
+	if (!strncmp(symbol, "sym.", 4)) {
+		symbol += 4;
+	}
+	if (!strncmp(symbol, "imp.", 4)) {
+		symbol += 4;
+	}
+	if (!strncmp(symbol, "target.", 7)) {
+		symbol += 7;
+	}
+	if (o) {
+		bool found = false;
+		rz_list_foreach (o->libs, iter, lib) {
+			size_t len = strlen(lib);
+			if (!rz_str_ncasecmp(symbol, lib, len)) {
+				symbol += len;
+				if (*symbol == '_') {
+					symbol++;
+				}
+				found = true;
+				break;
+			}
+		}
+		if (!found) {
+			lib = NULL;
+		}
+		size_t len = bin ? strlen(bin->file) : 0;
+		if (bin && len > 0 && !rz_str_ncasecmp(symbol, bin->file, len)) {
+			lib = bin->file;
+			symbol += len;
+			if (*symbol == '_') {
+				symbol++;
+			}
+		}
+	}
+
+	if (RZ_STR_ISEMPTY(symbol)) {
+		return NULL;
+	}
+
+	if (!strncmp(symbol, "__", 2)) {
+		if (symbol[2] == 'T') {
+			type = RZ_BIN_LANGUAGE_SWIFT;
+		} else {
+			type = RZ_BIN_LANGUAGE_CXX;
+		}
+	}
+
+	if (type == RZ_BIN_LANGUAGE_UNKNOWN) {
+		type = rz_bin_language_to_id(language);
+		// ignore "with blocks"
+		type = RZ_BIN_LANGUAGE_MASK(type);
+		language = rz_bin_language_to_string(type);
+	}
+	if (!language) {
+		return NULL;
+	}
+	char *demangled = NULL;
+	switch (type) {
+	case RZ_BIN_LANGUAGE_UNKNOWN: return NULL;
+	case RZ_BIN_LANGUAGE_KOTLIN:
+		/* fall-thru */
+	case RZ_BIN_LANGUAGE_GROOVY:
+		/* fall-thru */
+	case RZ_BIN_LANGUAGE_DART:
+		/* fall-thru */
+	case RZ_BIN_LANGUAGE_JAVA: demangled = rz_demangler_java(symbol); break;
+	case RZ_BIN_LANGUAGE_OBJC: demangled = rz_demangler_objc(symbol); break;
+	case RZ_BIN_LANGUAGE_MSVC: demangled = rz_demangler_msvc(symbol); break;
+#if WITH_GPL
+	case RZ_BIN_LANGUAGE_RUST: demangled = bin_demangle_rust(bf, symbol, vaddr); break;
+	case RZ_BIN_LANGUAGE_CXX: demangled = bin_demangle_cxx(bf, symbol, vaddr); break;
+#else
+	case RZ_BIN_LANGUAGE_RUST: demangled = NULL; break;
+	case RZ_BIN_LANGUAGE_CXX: demangled = NULL; break;
+#endif
+	default:
+		if (bin) {
+			rz_demangler_resolve(bin->demangler, symbol, language, &demangled);
+		}
+		break;
+	}
+	if (libs && demangled && lib) {
+		char *d = rz_str_newf("%s_%s", lib, demangled);
+		free(demangled);
+		demangled = d;
+	}
+	return demangled;
+}
