@@ -81,6 +81,10 @@ static DexString *dex_string_new(RzBuffer *buf, ut64 offset, st64 *pread) {
 	DexString *string = NULL;
 
 	read = rz_buf_uleb128(buf, &size);
+	if (UT64_ADD_OVFCHK(size, 1)) {
+		return NULL;
+	}
+
 	data = malloc(size + 1);
 	if (!data || rz_buf_read(buf, (ut8 *)data, size) != size) {
 		free(data);
@@ -816,6 +820,9 @@ static char *dex_resolve_proto_id(RzBinDex *dex, const char *name, ut32 proto_id
 	rz_strbuf_append(sb, "(");
 	for (ut32 i = 0; i < proto_id->type_list_size; ++i) {
 		ut32 type_idx = proto_id->type_list[i];
+		if (type_idx >= dex->type_ids_size) {
+			continue;
+		}
 
 		const DexString *param = dex_resolve_string_id_native(dex, dex->types[type_idx]);
 		if (!param) {
@@ -902,7 +909,9 @@ static char *dex_resolve_library(const char *library) {
 	}
 	char *demangled = strdup(library + 1);
 	rz_str_replace_ch(demangled, '/', '.', 1);
-	demangled[strlen(demangled) - 1] = 0;
+	if (RZ_STR_ISNOTEMPTY(demangled)) {
+		demangled[strlen(demangled) - 1] = 0;
+	}
 	return demangled;
 }
 
@@ -1429,7 +1438,8 @@ RZ_API RZ_OWN RzList /*<RzBinImport*>*/ *rz_bin_dex_imports(RZ_NONNULL RzBinDex
 		}
 
 		char *object = dex_resolve_type_id(dex, field_id->class_idx);
-		if (!object) {
+		if (RZ_STR_ISEMPTY(object)) {
+			free(object);
 			free(import);
 			break;
 		}
@@ -1476,7 +1486,8 @@ RZ_API RZ_OWN RzList /*<RzBinImport*>*/ *rz_bin_dex_imports(RZ_NONNULL RzBinDex
 		}
 
 		char *object = dex_resolve_type_id(dex, method_id->class_idx);
-		if (!object) {
+		if (RZ_STR_ISEMPTY(object)) {
+			free(object);
 			rz_bin_import_free(import);
 			break;
 		}
