@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Florian Märkl <info@florianmaerkl.de>
 // SPDX-FileCopyrightText: 2021 keegan
-// SPDX-FileCopyrightText: 2020 Francesco Tamagni <mrmacete@protonmail.ch>
+// SPDX-FileCopyrightText: 2020-2021 Francesco Tamagni <mrmacete@protonmail.ch>
 // SPDX-FileCopyrightText: 2018 pancake <pancake@nopcode.org>
 // SPDX-License-Identifier: LGPL-3.0-only
 
@@ -8,15 +8,24 @@
 
 #include <ht_pu.h>
 
-static RzDyldLocSym *rz_dyld_locsym_new(RzBuffer *cache_buf, cache_hdr_t *hdr);
+#define MAX_N_HDR 16
 
-static ut64 va2pa(uint64_t addr, cache_hdr_t *hdr, cache_map_t *maps, RzBuffer *cache_buf, ut64 slide, ut32 *offset, ut32 *left) {
+static RzDyldLocSym *rz_dyld_locsym_new(RzDyldCache *cache);
+
+/**
+ * \param magic zero-terminated string from the beginning of some file
+ */
+RZ_API bool rz_dyldcache_check_magic(const char *magic) {
+	return !strcmp(magic, "dyld_v1   arm64") || !strcmp(magic, "dyld_v1  arm64e") || !strcmp(magic, "dyld_v1  x86_64") || !strcmp(magic, "dyld_v1 x86_64h");
+}
+
+static ut64 va2pa(uint64_t addr, ut32 n_maps, cache_map_t *maps, RzBuffer *cache_buf, ut64 slide, ut32 *offset, ut32 *left) {
 	ut64 res = UT64_MAX;
-	uint32_t i;
+	ut32 i;
 
 	addr -= slide;
 
-	for (i = 0; i < hdr->mappingCount; i++) {
+	for (i = 0; i < n_maps; i++) {
 		if (addr >= maps[i].address && addr < maps[i].address + maps[i].size) {
 			res = maps[i].fileOffset + addr - maps[i].address;
 			if (offset) {
@@ -40,7 +49,7 @@ static void free_bin(RzDyldBinImage *bin) {
 	free(bin);
 }
 
-static cache_hdr_t *read_cache_header(RzBuffer *cache_buf) {
+static cache_hdr_t *read_cache_header(RzBuffer *cache_buf, ut64 offset) {
 	if (!cache_buf) {
 		return NULL;
 	}
@@ -51,31 +60,155 @@ static cache_hdr_t *read_cache_header(RzBuffer *cache_buf) {
 	}
 
 	ut64 size = sizeof(cache_hdr_t);
-	if (rz_buf_fread_at(cache_buf, 0, (ut8 *)hdr, "16c4i7l16clii4l", 1) != size) {
+	if (rz_buf_fread_at(cache_buf, offset, (ut8 *)hdr, "16c4i7l16clii4l", 1) != size) {
+		free(hdr);
+		return NULL;
+	}
+	if (!rz_dyldcache_check_magic(hdr->magic)) {
 		free(hdr);
 		return NULL;
 	}
 
+	if (!hdr->imagesCount && !hdr->imagesOffset) {
+		if (!rz_buf_read_le32_at(cache_buf, 0x1c0 + offset, &hdr->imagesOffset) || !rz_buf_read_le32_at(cache_buf, 0x1c4 + offset, &hdr->imagesCount)) {
+			free(hdr);
+			return NULL;
+		}
+	}
 	return hdr;
 }
 
-static cache_map_t *read_cache_maps(RzBuffer *cache_buf, cache_hdr_t *hdr) {
-	if (!cache_buf || !hdr || !hdr->mappingCount || !hdr->mappingOffset) {
-		return NULL;
+static void populate_cache_headers(RzDyldCache *cache) {
+	cache->n_hdr = 0;
+	RzList *hdrs = rz_list_newf(NULL);
+	if (!hdrs) {
+		return;
 	}
 
-	ut64 size = sizeof(cache_map_t) * hdr->mappingCount;
-	cache_map_t *maps = RZ_NEWS0(cache_map_t, hdr->mappingCount);
-	if (!maps) {
-		return NULL;
+	cache_hdr_t *h;
+	ut64 offsets[MAX_N_HDR];
+	ut64 offset = 0;
+	do {
+		offsets[cache->n_hdr] = offset;
+		h = read_cache_header(cache->buf, offset);
+		if (!h) {
+			break;
+		}
+		rz_list_append(hdrs, h);
+
+		ut64 size = h->codeSignatureOffset + h->codeSignatureSize;
+
+#define SHIFT_MAYBE(x) \
+	if (x) { \
+		x += offset; \
 	}
 
-	if (rz_buf_fread_at(cache_buf, hdr->mappingOffset, (ut8 *)maps, "3l2i", hdr->mappingCount) != size) {
-		RZ_FREE(maps);
-		return NULL;
+		SHIFT_MAYBE(h->mappingOffset);
+		SHIFT_MAYBE(h->imagesOffset);
+		SHIFT_MAYBE(h->codeSignatureOffset);
+		SHIFT_MAYBE(h->slideInfoOffset);
+		SHIFT_MAYBE(h->localSymbolsOffset);
+		SHIFT_MAYBE(h->branchPoolsOffset);
+		SHIFT_MAYBE(h->imagesTextOffset);
+
+		offset += size;
+		cache->n_hdr++;
+	} while (cache->n_hdr < MAX_N_HDR);
+
+	if (!cache->n_hdr) {
+		goto beach;
 	}
 
-	return maps;
+	cache->hdr = RZ_NEWS0(cache_hdr_t, cache->n_hdr);
+	if (!cache->hdr) {
+		cache->n_hdr = 0;
+		goto beach;
+	}
+
+	cache->hdr_offset = RZ_NEWS0(ut64, cache->n_hdr);
+	if (!cache->hdr_offset) {
+		cache->n_hdr = 0;
+		RZ_FREE(cache->hdr);
+		goto beach;
+	}
+
+	memcpy(cache->hdr_offset, offsets, cache->n_hdr * sizeof(ut64));
+
+	ut32 i = 0;
+	RzListIter *iter;
+	cache_hdr_t *item;
+	rz_list_foreach (hdrs, iter, item) {
+		if (i >= cache->n_hdr) {
+			break;
+		}
+		memcpy(&cache->hdr[i++], item, sizeof(cache_hdr_t));
+	}
+
+beach:
+	rz_list_free(hdrs);
+}
+
+static void populate_cache_maps(RzDyldCache *cache) {
+	rz_return_if_fail(cache && cache->buf);
+
+	ut32 i;
+	ut32 n_maps = 0;
+	ut64 max_count = 0;
+	for (i = 0; i < cache->n_hdr; i++) {
+		cache_hdr_t *hdr = &cache->hdr[i];
+		if (!hdr->mappingCount || !hdr->mappingOffset) {
+			continue;
+		}
+		max_count = RZ_MAX(hdr->mappingCount, max_count);
+		n_maps += hdr->mappingCount;
+	}
+
+	if (n_maps < 1 || n_maps < max_count /* overflow */) {
+		cache->maps = NULL;
+		cache->n_maps = 0;
+		return;
+	}
+
+	cache->maps_index = RZ_NEWS0(ut32, cache->n_hdr);
+	if (!cache->maps_index) {
+		return;
+	}
+	cache_map_t *maps = RZ_NEWS0(cache_map_t, n_maps);
+
+	ut32 next_map = 0;
+	ut32 last_idx = UT32_MAX;
+	ut64 max_address = 0;
+	for (i = 0; i < cache->n_hdr; i++) {
+		cache_hdr_t *hdr = &cache->hdr[i];
+		cache->maps_index[i] = next_map;
+
+		if (!hdr->mappingCount || !hdr->mappingOffset) {
+			continue;
+		}
+		ut64 size = sizeof(cache_map_t) * hdr->mappingCount;
+		if (rz_buf_fread_at(cache->buf, hdr->mappingOffset, (ut8 *)&maps[next_map], "3l2i", hdr->mappingCount) != size) {
+			continue;
+		}
+		ut32 j;
+		ut64 hdr_offset = cache->hdr_offset[i];
+		for (j = 0; j < hdr->mappingCount; j++) {
+			cache_map_t *map = &maps[next_map + j];
+			map->fileOffset += hdr_offset;
+			if (map->address > max_address) {
+				last_idx = i;
+				max_address = map->address;
+			}
+		}
+		next_map += hdr->mappingCount;
+	}
+
+	cache->maps = maps;
+	cache->n_maps = next_map;
+	if (last_idx == UT32_MAX) {
+		cache->symbols_off_base = 0;
+	} else {
+		cache->symbols_off_base = cache->hdr_offset[last_idx];
+	}
 }
 
 static cache_accel_t *read_cache_accel(RzBuffer *cache_buf, cache_hdr_t *hdr, cache_map_t *maps) {
@@ -83,7 +216,7 @@ static cache_accel_t *read_cache_accel(RzBuffer *cache_buf, cache_hdr_t *hdr, ca
 		return NULL;
 	}
 
-	ut64 offset = va2pa(hdr->accelerateInfoAddr, hdr, maps, cache_buf, 0, NULL, NULL);
+	ut64 offset = va2pa(hdr->accelerateInfoAddr, hdr->mappingCount, maps, cache_buf, 0, NULL, NULL);
 	if (!offset) {
 		return NULL;
 	}
@@ -111,8 +244,80 @@ static cache_accel_t *read_cache_accel(RzBuffer *cache_buf, cache_hdr_t *hdr, ca
 	return accel;
 }
 
-static cache_img_t *read_cache_images(RzBuffer *cache_buf, cache_hdr_t *hdr) {
-	if (!cache_buf || !hdr || !hdr->imagesCount || !hdr->imagesOffset) {
+objc_cache_opt_info *rz_dyldcache_get_objc_opt_info(RzBinFile *bf, RzDyldCache *cache) {
+	objc_cache_opt_info *result = NULL;
+	RzListIter *iter;
+	RzDyldBinImage *bin;
+	rz_list_foreach (cache->bins, iter, bin) {
+		if (!bin->file || strcmp(bin->file, "lib/libobjc.A.dylib")) {
+			continue;
+		}
+
+		struct MACH0_(opts_t) opts = { 0 };
+		opts.verbose = bf->rbin->verbose;
+		opts.header_at = bin->header_at;
+
+		struct MACH0_(obj_t) *mach0 = MACH0_(new_buf)(cache->buf, &opts);
+		if (!mach0) {
+			goto beach;
+		}
+
+		struct section_t *sections = NULL;
+		if (!(sections = MACH0_(get_sections)(mach0))) {
+			MACH0_(mach0_free)
+			(mach0);
+			goto beach;
+		}
+
+		int i;
+		ut64 scoffs_offset = 0;
+		ut64 scoffs_size = 0;
+		ut64 slide = rz_dyldcache_get_slide(cache);
+		for (i = 0; !sections[i].last; i++) {
+			if (sections[i].size == 0) {
+				continue;
+			}
+			if (strstr(sections[i].name, "__objc_scoffs")) {
+				scoffs_offset = va2pa(sections[i].addr, cache->n_maps, cache->maps, cache->buf, slide, NULL, NULL);
+				scoffs_size = sections[i].size;
+				break;
+			}
+		}
+
+		MACH0_(mach0_free)
+		(mach0);
+		RZ_FREE(sections);
+
+		if (!scoffs_offset || scoffs_size < 40) {
+			break;
+		}
+		ut64 check;
+		if (!rz_buf_read_le64_at(cache->buf, scoffs_offset, &check) || check != 2) {
+			break;
+		}
+		ut64 sel_string_base;
+		if (!rz_buf_read_le64_at(cache->buf, scoffs_offset + 8, &sel_string_base)) {
+			break;
+		}
+		ut64 sel_string_end;
+		if (!rz_buf_read_le64_at(cache->buf, scoffs_offset + 16, &sel_string_end) || sel_string_end == sel_string_base) {
+			break;
+		}
+		result = RZ_NEW0(objc_cache_opt_info);
+		if (!result) {
+			break;
+		}
+		result->sel_string_base = sel_string_base;
+	}
+beach:
+	return result;
+}
+
+static cache_img_t *read_cache_images(RzBuffer *cache_buf, cache_hdr_t *hdr, ut64 hdr_offset) {
+	if (!cache_buf || !hdr) {
+		return NULL;
+	}
+	if (!hdr->imagesCount || !hdr->imagesOffset || hdr->imagesOffset == UT32_MAX || hdr->imagesCount == UT32_MAX) {
 		return NULL;
 	}
 
@@ -127,9 +332,57 @@ static cache_img_t *read_cache_images(RzBuffer *cache_buf, cache_hdr_t *hdr) {
 		return NULL;
 	}
 
+	if (hdr_offset) {
+		ut32 i;
+		for (i = 0; i < hdr->imagesCount; i++) {
+			cache_img_t *img = &images[i];
+			img->pathFileOffset += hdr_offset;
+		}
+	}
+
 	return images;
 }
 
+static void match_bin_entries(RzDyldCache *cache, void *entries) {
+	rz_return_if_fail(cache && cache->bins && entries);
+
+	cache_img_t *imgs = read_cache_images(cache->buf, cache->hdr, 0);
+	if (!imgs) {
+		return;
+	}
+
+	RzDyldBinImage *bin = NULL;
+	RzListIter *it = rz_list_iterator(cache->bins);
+
+	bool has_large_entries = cache->n_hdr > 1;
+
+	ut32 i;
+	for (i = 0; i < cache->hdr->imagesCount; i++) {
+		cache_img_t *img = &imgs[i];
+		if (!it) {
+			break;
+		}
+		bin = it->data;
+		if (!bin) {
+			break;
+		}
+		if (bin && bin->va == img->address) {
+			if (has_large_entries) {
+				cache_locsym_entry_large_t *e = &((cache_locsym_entry_large_t *)entries)[i];
+				bin->nlist_start_index = e->nlistStartIndex;
+				bin->nlist_count = e->nlistCount;
+			} else {
+				cache_locsym_entry_t *e = &((cache_locsym_entry_t *)entries)[i];
+				bin->nlist_start_index = e->nlistStartIndex;
+				bin->nlist_count = e->nlistCount;
+			}
+			it = it->n;
+		}
+	}
+
+	RZ_FREE(imgs);
+}
+
 static cache_imgxtr_t *read_cache_imgextra(RzBuffer *cache_buf, cache_hdr_t *hdr, cache_accel_t *accel) {
 	if (!cache_buf || !hdr || !hdr->imagesCount || !accel || !accel->imageExtrasCount || !accel->imagesExtrasOffset) {
 		return NULL;
@@ -180,13 +433,13 @@ static HtPU *create_path_to_index(RzBuffer *cache_buf, cache_img_t *img, cache_h
 	return path_to_idx;
 }
 
-static void carve_deps_at_address(RzBuffer *cache_buf, cache_img_t *img, cache_hdr_t *hdr, cache_map_t *maps, HtPU *path_to_idx, ut64 address, int *deps) {
-	ut64 pa = va2pa(address, hdr, maps, cache_buf, 0, NULL, NULL);
+static void carve_deps_at_address(RzDyldCache *cache, cache_img_t *img, HtPU *path_to_idx, ut64 address, int *deps, bool printing) {
+	ut64 pa = va2pa(address, cache->n_maps, cache->maps, cache->buf, 0, NULL, NULL);
 	if (pa == UT64_MAX) {
 		return;
 	}
 	struct MACH0_(mach_header) mh;
-	if (rz_buf_fread_at(cache_buf, pa, (ut8 *)&mh, "8i", 1) != sizeof(struct MACH0_(mach_header))) {
+	if (rz_buf_fread_at(cache->buf, pa, (ut8 *)&mh, "8i", 1) != sizeof(struct MACH0_(mach_header))) {
 		return;
 	}
 	if (mh.magic != MH_MAGIC_64 || mh.sizeofcmds == 0) {
@@ -194,7 +447,7 @@ static void carve_deps_at_address(RzBuffer *cache_buf, cache_img_t *img, cache_h
 	}
 	ut64 cmds_at = pa + sizeof(struct MACH0_(mach_header));
 	ut8 *cmds = malloc(mh.sizeofcmds + 1);
-	if (!cmds || rz_buf_read_at(cache_buf, cmds_at, cmds, mh.sizeofcmds) != mh.sizeofcmds) {
+	if (!cmds || rz_buf_read_at(cache->buf, cmds_at, cmds, mh.sizeofcmds) != mh.sizeofcmds) {
 		goto beach;
 	}
 	cmds[mh.sizeofcmds] = 0;
@@ -213,12 +466,14 @@ static void carve_deps_at_address(RzBuffer *cache_buf, cache_img_t *img, cache_h
 			}
 			const char *key = (const char *)cursor + 24;
 			size_t dep_index = (size_t)ht_pu_find(path_to_idx, key, &found);
-			if (!found || dep_index >= hdr->imagesCount) {
-				eprintf("WARNING: alien dep '%s'\n", key);
+			if (!found || dep_index >= cache->hdr->imagesCount) {
+				RZ_LOG_WARN("alien dep '%s'\n", key);
 				continue;
 			}
 			deps[dep_index]++;
-			eprintf("-> %s\n", key);
+			if (printing) {
+				RZ_LOG_INFO("-> %s\n", key);
+			}
 		}
 		cursor += cmdsize;
 	}
@@ -227,164 +482,173 @@ beach:
 	free(cmds);
 }
 
-static RzList *create_cache_bins(RzBuffer *cache_buf, cache_hdr_t *hdr, cache_map_t *maps, cache_accel_t *accel) {
+static RzList *create_cache_bins(RzDyldCache *cache) {
 	RzList *bins = rz_list_newf((RzListFree)free_bin);
 	if (!bins) {
 		return NULL;
 	}
 
-	cache_img_t *img = read_cache_images(cache_buf, hdr);
-	if (!img) {
-		rz_list_free(bins);
-		return NULL;
-	}
-
-	int i;
-	int *deps = NULL;
 	char *target_libs = NULL;
-	target_libs = rz_sys_getenv("RZ_DYLDCACHE_FILTER");
 	RzList *target_lib_names = NULL;
-	ut16 *depArray = NULL;
-	cache_imgxtr_t *extras = NULL;
+	int *deps = NULL;
+	target_libs = rz_sys_getenv("RZ_DYLDCACHE_FILTER");
 	if (target_libs) {
 		target_lib_names = rz_str_split_list(target_libs, ":", 0);
 		if (!target_lib_names) {
-			goto error;
+			rz_list_free(bins);
+			return NULL;
 		}
-
-		deps = RZ_NEWS0(int, hdr->imagesCount);
+		deps = RZ_NEWS0(int, cache->hdr->imagesCount);
 		if (!deps) {
-			goto error;
+			rz_list_free(bins);
+			rz_list_free(target_lib_names);
+			return NULL;
 		}
-
-		HtPU *path_to_idx = NULL;
-		if (accel) {
-			depArray = RZ_NEWS0(ut16, accel->depListCount);
-			if (!depArray) {
-				goto error;
-			}
-
-			if (rz_buf_fread_at(cache_buf, accel->depListOffset, (ut8 *)depArray, "s", accel->depListCount) != accel->depListCount * 2) {
-				goto error;
-			}
-
-			extras = read_cache_imgextra(cache_buf, hdr, accel);
-			if (!extras) {
-				goto error;
-			}
-		} else {
-			path_to_idx = create_path_to_index(cache_buf, img, hdr);
-		}
-
-		for (i = 0; i < hdr->imagesCount; i++) {
-			char *lib_name = get_lib_name(cache_buf, &img[i]);
-			if (!lib_name) {
-				break;
-			}
-			if (strstr(lib_name, "libobjc.A.dylib")) {
-				deps[i]++;
-			}
-			if (!rz_list_find(target_lib_names, lib_name, string_contains)) {
-				RZ_FREE(lib_name);
-				continue;
-			}
-			eprintf("FILTER: %s\n", lib_name);
-			RZ_FREE(lib_name);
-			deps[i]++;
-
-			if (extras && depArray) {
-				ut32 j;
-				for (j = extras[i].dependentsStartArrayIndex; depArray[j] != 0xffff; j++) {
-					ut16 dep_index = depArray[j] & 0x7fff;
-					deps[dep_index]++;
-
-					char *dep_name = get_lib_name(cache_buf, &img[dep_index]);
-					if (!dep_name) {
-						break;
-					}
-					eprintf("-> %s\n", dep_name);
-					free(dep_name);
-				}
-			} else if (path_to_idx) {
-				carve_deps_at_address(cache_buf, img, hdr, maps, path_to_idx, img[i].address, deps);
-			}
-		}
-
-		ht_pu_free(path_to_idx);
-		RZ_FREE(depArray);
-		RZ_FREE(extras);
-		RZ_FREE(target_libs);
-		rz_list_free(target_lib_names);
-		target_lib_names = NULL;
 	}
 
-	for (i = 0; i < hdr->imagesCount; i++) {
-		if (deps && !deps[i]) {
-			continue;
+	ut32 i;
+	for (i = 0; i < cache->n_hdr; i++) {
+		cache_hdr_t *hdr = &cache->hdr[i];
+		ut64 hdr_offset = cache->hdr_offset[i];
+		ut64 symbols_off = cache->symbols_off_base - hdr_offset;
+		ut32 maps_index = cache->maps_index[i];
+		cache_img_t *img = read_cache_images(cache->buf, hdr, hdr_offset);
+		if (!img) {
+			goto next;
 		}
-		ut64 pa = va2pa(img[i].address, hdr, maps, cache_buf, 0, NULL, NULL);
-		if (pa == UT64_MAX) {
-			continue;
-		}
-		ut8 magicbytes[4];
-		rz_buf_read_at(cache_buf, pa, magicbytes, 4);
-		int magic = rz_read_le32(magicbytes);
-		switch (magic) {
-		case MH_MAGIC:
-			// 32bit not supported (yet)
-			break;
-		case MH_MAGIC_64: {
-			char file[256];
-			RzDyldBinImage *bin = RZ_NEW0(RzDyldBinImage);
-			if (!bin) {
-				goto error;
+
+		ut32 j;
+		ut16 *depArray = NULL;
+		cache_imgxtr_t *extras = NULL;
+		if (target_libs) {
+			HtPU *path_to_idx = NULL;
+			if (cache->accel) {
+				depArray = RZ_NEWS0(ut16, cache->accel->depListCount);
+				if (!depArray) {
+					goto next;
+				}
+
+				if (rz_buf_fread_at(cache->buf, cache->accel->depListOffset, (ut8 *)depArray, "s", cache->accel->depListCount) != cache->accel->depListCount * 2) {
+					goto next;
+				}
+
+				extras = read_cache_imgextra(cache->buf, hdr, cache->accel);
+				if (!extras) {
+					goto next;
+				}
+			} else {
+				path_to_idx = create_path_to_index(cache->buf, img, hdr);
 			}
-			bin->header_at = pa;
-			if (rz_buf_read_at(cache_buf, img[i].pathFileOffset, (ut8 *)&file, sizeof(file)) == sizeof(file)) {
-				file[255] = 0;
-				char *last_slash = strrchr(file, '/');
-				if (last_slash && *last_slash) {
-					if (last_slash > file) {
-						char *scan = last_slash - 1;
-						while (scan > file && *scan != '/') {
-							scan--;
+
+			for (j = 0; j < hdr->imagesCount; j++) {
+				bool printing = !deps[j];
+				char *lib_name = get_lib_name(cache->buf, &img[j]);
+				if (!lib_name) {
+					break;
+				}
+				if (strstr(lib_name, "libobjc.A.dylib")) {
+					deps[j]++;
+				}
+				if (!rz_list_find(target_lib_names, lib_name, string_contains)) {
+					RZ_FREE(lib_name);
+					continue;
+				}
+				if (printing) {
+					RZ_LOG_INFO("FILTER: %s\n", lib_name);
+				}
+				RZ_FREE(lib_name);
+				deps[j]++;
+
+				if (extras && depArray) {
+					ut32 k;
+					for (k = extras[j].dependentsStartArrayIndex; depArray[k] != 0xffff; k++) {
+						ut16 dep_index = depArray[k] & 0x7fff;
+						deps[dep_index]++;
+
+						char *dep_name = get_lib_name(cache->buf, &img[dep_index]);
+						if (!dep_name) {
+							break;
 						}
-						if (*scan == '/') {
-							bin->file = strdup(scan + 1);
+						if (printing) {
+							RZ_LOG_INFO("-> %s\n", dep_name);
+						}
+						free(dep_name);
+					}
+				} else if (path_to_idx) {
+					carve_deps_at_address(cache, img, path_to_idx, img[j].address, deps, printing);
+				}
+			}
+
+			ht_pu_free(path_to_idx);
+			RZ_FREE(depArray);
+			RZ_FREE(extras);
+		}
+
+		for (j = 0; j < hdr->imagesCount; j++) {
+			if (deps && !deps[j]) {
+				continue;
+			}
+			ut64 pa = va2pa(img[j].address, hdr->mappingCount, &cache->maps[maps_index], cache->buf, 0, NULL, NULL);
+			if (pa == UT64_MAX) {
+				continue;
+			}
+			ut8 magicbytes[4];
+			rz_buf_read_at(cache->buf, pa, magicbytes, 4);
+			int magic = rz_read_le32(magicbytes);
+			switch (magic) {
+			case MH_MAGIC_64: {
+				char file[256];
+				RzDyldBinImage *bin = RZ_NEW0(RzDyldBinImage);
+				if (!bin) {
+					goto next;
+				}
+				bin->header_at = pa;
+				bin->hdr_offset = hdr_offset;
+				bin->symbols_off = symbols_off;
+				bin->va = img[j].address;
+				if (rz_buf_read_at(cache->buf, img[j].pathFileOffset, (ut8 *)&file, sizeof(file)) == sizeof(file)) {
+					file[255] = 0;
+					char *last_slash = strrchr(file, '/');
+					if (last_slash && *last_slash) {
+						if (last_slash > file) {
+							char *scan = last_slash - 1;
+							while (scan > file && *scan != '/') {
+								scan--;
+							}
+							if (*scan == '/') {
+								bin->file = strdup(scan + 1);
+							} else {
+								bin->file = strdup(last_slash + 1);
+							}
 						} else {
 							bin->file = strdup(last_slash + 1);
 						}
 					} else {
-						bin->file = strdup(last_slash + 1);
+						bin->file = strdup(file);
 					}
 				} else {
-					bin->file = strdup(file);
+					bin->file = rz_str_newf("unknown_image_%08" PFMT64x, symbols_off);
 				}
+				rz_list_append(bins, bin);
+				break;
+			}
+			default:
+				RZ_LOG_WARN("Unknown sub-bin\n");
+				break;
 			}
-			rz_list_append(bins, bin);
-			break;
-		}
-		default:
-			eprintf("Unknown sub-bin\n");
-			break;
 		}
+	next:
+		RZ_FREE(depArray);
+		RZ_FREE(extras);
+		RZ_FREE(img);
 	}
-
-	goto beach;
-error:
-	if (bins) {
+	if (rz_list_empty(bins)) {
 		rz_list_free(bins);
-	}
-	bins = NULL;
-beach:
-	RZ_FREE(depArray);
-	RZ_FREE(extras);
-	RZ_FREE(target_libs);
-	if (target_lib_names) {
-		rz_list_free(target_lib_names);
+		bins = NULL;
 	}
 	RZ_FREE(deps);
-	RZ_FREE(img);
+	RZ_FREE(target_libs);
+	rz_list_free(target_lib_names);
 	return bins;
 }
 
@@ -411,6 +675,9 @@ static ut32 dumb_ctzll(ut64 x) {
 
 static ut64 estimate_slide(RzDyldCache *cache, ut64 value_mask, ut64 value_add) {
 	ut64 slide = 0;
+	if (cache->n_hdr > 1) {
+		return slide;
+	}
 	ut64 *classlist = malloc(64);
 	if (!classlist) {
 		goto beach;
@@ -461,8 +728,9 @@ static ut64 estimate_slide(RzDyldCache *cache, ut64 value_mask, ut64 value_add)
 
 		int classlist_sample_size = RZ_MIN(64, sections[classlist_idx].size);
 		int n_classes = classlist_sample_size / 8;
+		ut64 sect_offset = sections[classlist_idx].offset + bin->hdr_offset;
 
-		if (rz_buf_fread_at(cache->buf, sections[classlist_idx].offset, (ut8 *)classlist, "l", n_classes) < classlist_sample_size) {
+		if (rz_buf_fread_at(cache->buf, sect_offset, (ut8 *)classlist, "l", n_classes) < classlist_sample_size) {
 			goto next_bin;
 		}
 
@@ -555,7 +823,7 @@ static RzDyldRebaseInfo *get_rebase_info(RzDyldCache *cache, ut64 slideInfoOffse
 		if (slide == UT64_MAX) {
 			rebase_info->slide = estimate_slide(cache, 0x7ffffffffffffULL, 0);
 			if (rebase_info->slide) {
-				eprintf("dyldcache is slid: 0x%" PFMT64x "\n", rebase_info->slide);
+				RZ_LOG_INFO("dyldcache is slid: 0x%" PFMT64x "\n", rebase_info->slide);
 			}
 		} else {
 			rebase_info->slide = slide;
@@ -632,7 +900,7 @@ static RzDyldRebaseInfo *get_rebase_info(RzDyldCache *cache, ut64 slideInfoOffse
 		if (slide == UT64_MAX) {
 			rebase_info->slide = estimate_slide(cache, rebase_info->value_mask, rebase_info->value_add);
 			if (rebase_info->slide) {
-				eprintf("dyldcache is slid: 0x%" PFMT64x "\n", rebase_info->slide);
+				RZ_LOG_INFO("dyldcache is slid: 0x%" PFMT64x "\n", rebase_info->slide);
 			}
 		} else {
 			rebase_info->slide = slide;
@@ -703,7 +971,7 @@ static RzDyldRebaseInfo *get_rebase_info(RzDyldCache *cache, ut64 slideInfoOffse
 		if (slide == UT64_MAX) {
 			rebase_info->slide = estimate_slide(cache, UT64_MAX, 0);
 			if (rebase_info->slide) {
-				eprintf("dyldcache is slid: 0x%" PFMT64x "\n", rebase_info->slide);
+				RZ_LOG_INFO("dyldcache is slid: 0x%" PFMT64x "\n", rebase_info->slide);
 			}
 		} else {
 			rebase_info->slide = slide;
@@ -711,7 +979,7 @@ static RzDyldRebaseInfo *get_rebase_info(RzDyldCache *cache, ut64 slideInfoOffse
 
 		return (RzDyldRebaseInfo *)rebase_info;
 	} else {
-		eprintf("unsupported slide info version %d\n", slide_info_version);
+		RZ_LOG_ERROR("Unsupported slide info version %d\n", slide_info_version);
 		return NULL;
 	}
 
@@ -723,68 +991,93 @@ beach:
 }
 
 static RzDyldRebaseInfos *get_rebase_infos(RzDyldCache *cache) {
-	RzBuffer *cache_buf = cache->buf;
-
 	RzDyldRebaseInfos *result = RZ_NEW0(RzDyldRebaseInfos);
 	if (!result) {
 		return NULL;
 	}
 
 	if (!cache->hdr->slideInfoOffset || !cache->hdr->slideInfoSize) {
-		ut32 slide_infos_offset;
-		if (!rz_buf_read_le32_at(cache_buf, 0x138, &slide_infos_offset)) {
+		size_t total_slide_infos = 0;
+		ut32 n_slide_infos[MAX_N_HDR];
+
+		ut32 i;
+		for (i = 0; i < cache->n_hdr && i < MAX_N_HDR; i++) {
+			ut64 hdr_offset = cache->hdr_offset[i];
+			if (!rz_buf_read_le32_at(cache->buf, 0x13c + hdr_offset, &n_slide_infos[i])) {
+				goto beach;
+			}
+			ut32 total = total_slide_infos + n_slide_infos[i];
+			if (total < total_slide_infos) {
+				// overflow
+				goto beach;
+			}
+			total_slide_infos = total;
+		}
+
+		if (!total_slide_infos) {
 			goto beach;
 		}
 
-		ut32 n_slide_infos;
-		if (!rz_buf_read_le32_at(cache_buf, 0x13c, &n_slide_infos)) {
-			goto beach;
-		}
-
-		RzDyldRebaseInfosEntry *infos = RZ_NEWS0(RzDyldRebaseInfosEntry, n_slide_infos);
+		RzDyldRebaseInfosEntry *infos = RZ_NEWS0(RzDyldRebaseInfosEntry, total_slide_infos);
 		if (!infos) {
 			goto beach;
 		}
 
-		size_t i, j;
-		RzDyldRebaseInfo *prev_info = NULL;
-		for (i = 0, j = 0; i < n_slide_infos; i++) {
-			ut64 offset = slide_infos_offset + i * sizeof(cache_mapping_slide);
-			cache_mapping_slide entry;
-			if (rz_buf_fread_at(cache_buf, offset, (ut8 *)&entry, "6lii", 1) != sizeof(cache_mapping_slide)) {
-				free(infos);
-				goto beach;
+		ut32 k = 0;
+		for (i = 0; i < cache->n_hdr && i < MAX_N_HDR; i++) {
+			ut64 hdr_offset = cache->hdr_offset[i];
+			if (!n_slide_infos[i]) {
+				continue;
 			}
+			ut32 sio;
+			if (!rz_buf_read_le32_at(cache->buf, 0x138 + hdr_offset, &sio)) {
+				continue;
+			}
+			ut64 slide_infos_offset = sio;
+			if (!slide_infos_offset) {
+				continue;
+			}
+			slide_infos_offset += hdr_offset;
 
-			if (entry.slideInfoOffset && entry.slideInfoSize) {
-				infos[j].start = entry.fileOffset;
-				infos[j].end = entry.fileOffset + entry.size;
-				ut64 slide = prev_info ? prev_info->slide : UT64_MAX;
-				infos[j].info = get_rebase_info(cache, entry.slideInfoOffset, entry.slideInfoSize, entry.fileOffset, slide);
-				prev_info = infos[j].info;
-				j++;
+			ut32 j;
+			RzDyldRebaseInfo *prev_info = NULL;
+			for (j = 0; j < n_slide_infos[i]; j++) {
+				ut64 offset = slide_infos_offset + j * sizeof(cache_mapping_slide);
+				cache_mapping_slide entry;
+				if (rz_buf_fread_at(cache->buf, offset, (ut8 *)&entry, "6lii", 1) != sizeof(cache_mapping_slide)) {
+					break;
+				}
+
+				if (entry.slideInfoOffset && entry.slideInfoSize) {
+					infos[k].start = entry.fileOffset + hdr_offset;
+					infos[k].end = infos[k].start + entry.size;
+					ut64 slide = prev_info ? prev_info->slide : UT64_MAX;
+					infos[k].info = get_rebase_info(cache, entry.slideInfoOffset + hdr_offset, entry.slideInfoSize, entry.fileOffset + hdr_offset, slide);
+					prev_info = infos[k].info;
+					k++;
+				}
 			}
 		}
 
-		if (!j) {
+		if (!k) {
 			free(infos);
 			goto beach;
 		}
 
-		if (j != n_slide_infos) {
-			RzDyldRebaseInfosEntry *pruned_infos = RZ_NEWS0(RzDyldRebaseInfosEntry, j);
+		if (k < total_slide_infos) {
+			RzDyldRebaseInfosEntry *pruned_infos = RZ_NEWS0(RzDyldRebaseInfosEntry, k);
 			if (!pruned_infos) {
 				free(infos);
 				goto beach;
 			}
 
-			memcpy(pruned_infos, infos, sizeof(RzDyldRebaseInfosEntry) * j);
+			memcpy(pruned_infos, infos, sizeof(RzDyldRebaseInfosEntry) * k);
 			free(infos);
 			infos = pruned_infos;
 		}
 
 		result->entries = infos;
-		result->length = j;
+		result->length = k;
 		return result;
 	}
 
@@ -824,6 +1117,63 @@ RZ_API ut64 rz_dyldcache_get_slide(RzDyldCache *cache) {
 	return 0;
 }
 
+RZ_API void rz_dyldcache_symbols_from_locsym(RzDyldCache *cache, RzDyldBinImage *bin, RzList *symbols, SetU *hash) {
+	RzDyldLocSym *locsym = cache->locsym;
+	if (!locsym) {
+		return;
+	}
+
+	if (bin->nlist_start_index >= locsym->nlists_count ||
+		bin->nlist_start_index + bin->nlist_count > locsym->nlists_count) {
+		RZ_LOG_ERROR("dyldcache: malformed local symbol entry\n");
+		return;
+	}
+
+	ut64 nlists_size = sizeof(struct MACH0_(nlist)) * bin->nlist_count;
+	struct MACH0_(nlist) *nlists = RZ_NEWS0(struct MACH0_(nlist), bin->nlist_count);
+	if (!nlists) {
+		return;
+	}
+	ut64 nlists_offset = locsym->local_symbols_offset + locsym->nlists_offset +
+		bin->nlist_start_index * sizeof(struct MACH0_(nlist));
+	if (rz_buf_fread_at(cache->buf, nlists_offset, (ut8 *)nlists, "iccsl", bin->nlist_count) != nlists_size) {
+		free(nlists);
+		return;
+	}
+
+	ut32 j;
+	for (j = 0; j != bin->nlist_count; j++) {
+		struct MACH0_(nlist) *nlist = &nlists[j];
+		if (set_u_contains(hash, (ut64)nlist->n_value)) {
+			continue;
+		}
+		set_u_add(hash, (ut64)nlist->n_value);
+		if (nlist->n_strx >= locsym->strings_size) {
+			continue;
+		}
+		RzBinSymbol *sym = RZ_NEW0(RzBinSymbol);
+		if (!sym) {
+			break;
+		}
+		sym->type = "LOCAL";
+		sym->vaddr = nlist->n_value;
+		ut64 slide = rz_dyldcache_get_slide(cache);
+		sym->paddr = va2pa(nlist->n_value, cache->n_maps, cache->maps, cache->buf, slide, NULL, NULL);
+
+		char *symstr = rz_buf_get_string(cache->buf, locsym->local_symbols_offset + locsym->strings_offset + nlist->n_strx);
+		if (symstr) {
+			sym->name = symstr;
+		} else {
+			static ut32 k = 0;
+			sym->name = rz_str_newf("unk_local%d", k++);
+		}
+
+		rz_list_append(symbols, sym);
+	}
+
+	free(nlists);
+}
+
 RZ_API RzDyldCache *rz_dyldcache_new_buf(RzBuffer *buf) {
 	RzDyldCache *cache = RZ_NEW0(RzDyldCache);
 	if (!cache) {
@@ -831,20 +1181,20 @@ RZ_API RzDyldCache *rz_dyldcache_new_buf(RzBuffer *buf) {
 	}
 	memcpy(cache->magic, "dyldcac", 7);
 	cache->buf = rz_buf_ref(buf);
-	cache->hdr = read_cache_header(cache->buf);
+	populate_cache_headers(cache);
 	if (!cache->hdr) {
 		goto cupertino;
 	}
-	cache->maps = read_cache_maps(cache->buf, cache->hdr);
+	populate_cache_maps(cache);
 	if (!cache->maps) {
 		goto cupertino;
 	}
 	cache->accel = read_cache_accel(cache->buf, cache->hdr, cache->maps);
-	cache->locsym = rz_dyld_locsym_new(cache->buf, cache->hdr);
-	cache->bins = create_cache_bins(cache->buf, cache->hdr, cache->maps, cache->accel);
+	cache->bins = create_cache_bins(cache);
 	if (!cache->bins) {
 		goto cupertino;
 	}
+	cache->locsym = rz_dyld_locsym_new(cache);
 	cache->rebase_infos = get_rebase_infos(cache);
 	return cache;
 cupertino:
@@ -898,90 +1248,87 @@ static void rebase_info_free(RzDyldRebaseInfo *rebase_info) {
 	}
 }
 
-static RzDyldLocSym *rz_dyld_locsym_new(RzBuffer *cache_buf, cache_hdr_t *hdr) {
-	if (!cache_buf || !hdr || !hdr->localSymbolsSize || !hdr->localSymbolsOffset) {
-		return NULL;
+static RzDyldLocSym *rz_dyld_locsym_new(RzDyldCache *cache) {
+	rz_return_val_if_fail(cache && cache->buf, NULL);
+
+	ut32 i;
+	for (i = 0; i < cache->n_hdr; i++) {
+		cache_hdr_t *hdr = &cache->hdr[i];
+		if (!hdr || !hdr->localSymbolsSize || !hdr->localSymbolsOffset) {
+			continue;
+		}
+
+		cache_locsym_info_t *info = NULL;
+		void *entries = NULL;
+
+		ut64 info_size = sizeof(cache_locsym_info_t);
+		info = RZ_NEW0(cache_locsym_info_t);
+		if (!info) {
+			goto beach;
+		}
+		if (rz_buf_fread_at(cache->buf, hdr->localSymbolsOffset, (ut8 *)info, "6i", 1) != info_size) {
+			RZ_LOG_ERROR("Cannot read cache_locsym_info_t from header\n");
+			goto beach;
+		}
+		if (info->entriesCount != cache->hdr->imagesCount) {
+			RZ_LOG_ERROR("The number of entries count differs from cache header image count\n");
+			goto beach;
+		}
+
+		bool has_large_entries = cache->n_hdr > 1;
+		if (has_large_entries) {
+			ut64 entries_size = sizeof(cache_locsym_entry_large_t) * info->entriesCount;
+			cache_locsym_entry_large_t *large_entries = RZ_NEWS0(cache_locsym_entry_large_t, info->entriesCount);
+			if (!large_entries) {
+				goto beach;
+			}
+			if (rz_buf_fread_at(cache->buf, hdr->localSymbolsOffset + info->entriesOffset, (ut8 *)large_entries, "lii",
+				    info->entriesCount) != entries_size) {
+				RZ_LOG_ERROR("Cannot read cache_locsym_entry_large_t\n");
+				goto beach;
+			}
+			entries = large_entries;
+		} else {
+			ut64 entries_size = sizeof(cache_locsym_entry_t) * info->entriesCount;
+			cache_locsym_entry_t *regular_entries = RZ_NEWS0(cache_locsym_entry_t, info->entriesCount);
+			if (!regular_entries) {
+				goto beach;
+			}
+			if (rz_buf_fread_at(cache->buf, hdr->localSymbolsOffset + info->entriesOffset, (ut8 *)regular_entries, "iii",
+				    info->entriesCount) != entries_size) {
+				RZ_LOG_ERROR("Cannot read cache_locsym_entry_t\n");
+				goto beach;
+			}
+			entries = regular_entries;
+		}
+		RzDyldLocSym *locsym = RZ_NEW0(RzDyldLocSym);
+		if (!locsym) {
+			goto beach;
+		}
+
+		match_bin_entries(cache, entries);
+
+		locsym->local_symbols_offset = hdr->localSymbolsOffset;
+		locsym->nlists_offset = info->nlistOffset;
+		locsym->nlists_count = info->nlistCount;
+		locsym->strings_offset = info->stringsOffset;
+		locsym->strings_size = info->stringsSize;
+
+		free(info);
+		free(entries);
+
+		return locsym;
+
+	beach:
+		free(info);
+		free(entries);
+
+		RZ_LOG_ERROR("dyldcache: malformed local symbols metadata\n");
+		break;
 	}
-
-	cache_locsym_info_t *info = NULL;
-	char *strings = NULL;
-	cache_locsym_entry_t *entries = NULL;
-	struct MACH0_(nlist) *nlists = NULL;
-
-	ut64 info_size = sizeof(cache_locsym_info_t);
-	info = RZ_NEW0(cache_locsym_info_t);
-	if (!info) {
-		goto beach;
-	}
-	if (rz_buf_fread_at(cache_buf, hdr->localSymbolsOffset, (ut8 *)info, "6i", 1) != info_size) {
-		goto beach;
-	}
-
-	ut64 nlists_size = sizeof(struct MACH0_(nlist)) * info->nlistCount;
-	nlists = RZ_NEWS0(struct MACH0_(nlist), info->nlistCount);
-	if (!nlists) {
-		goto beach;
-	}
-	if (rz_buf_fread_at(cache_buf, hdr->localSymbolsOffset + info->nlistOffset, (ut8 *)nlists, "iccsl",
-		    info->nlistCount) != nlists_size) {
-		goto beach;
-	}
-
-	strings = malloc(info->stringsSize);
-	if (!strings) {
-		goto beach;
-	}
-	if (rz_buf_read_at(cache_buf, hdr->localSymbolsOffset + info->stringsOffset, (ut8 *)strings,
-		    info->stringsSize) != info->stringsSize) {
-		goto beach;
-	}
-
-	ut64 entries_size = sizeof(cache_locsym_entry_t) * info->entriesCount;
-	entries = RZ_NEWS0(cache_locsym_entry_t, info->entriesCount);
-	if (!entries) {
-		goto beach;
-	}
-	if (rz_buf_fread_at(cache_buf, hdr->localSymbolsOffset + info->entriesOffset, (ut8 *)entries, "3i",
-		    info->entriesCount) != entries_size) {
-		goto beach;
-	}
-
-	RzDyldLocSym *locsym = RZ_NEW0(RzDyldLocSym);
-	if (!locsym) {
-		goto beach;
-	}
-
-	locsym->nlists = nlists;
-	locsym->nlists_count = info->nlistCount;
-	locsym->strings = strings;
-	locsym->strings_size = info->stringsSize;
-	locsym->entries = entries;
-	locsym->entries_count = info->entriesCount;
-
-	free(info);
-
-	return locsym;
-
-beach:
-	free(info);
-	free(strings);
-	free(entries);
-	free(nlists);
-
-	eprintf("dyldcache: malformed local symbols metadata\n");
 	return NULL;
 }
 
-static void rz_dyld_locsym_free(RzDyldLocSym *locsym) {
-	if (!locsym) {
-		return;
-	}
-	free(locsym->strings);
-	free(locsym->entries);
-	free(locsym->nlists);
-	free(locsym);
-}
-
 RZ_API void rz_dyldcache_free(RzDyldCache *cache) {
 	if (!cache) {
 		return;
@@ -1002,76 +1349,20 @@ RZ_API void rz_dyldcache_free(RzDyldCache *cache) {
 	}
 	free(cache->hdr);
 	free(cache->maps);
+	free(cache->maps_index);
+	free(cache->hdr_offset);
 	free(cache->accel);
-	rz_dyld_locsym_free(cache->locsym);
+	free(cache->locsym);
+	free(cache->oi);
 	free(cache);
 }
 
 RZ_API ut64 rz_dyldcache_va2pa(RzDyldCache *cache, uint64_t vaddr, ut32 *offset, ut32 *left) {
 	rz_return_val_if_fail(cache, UT64_MAX);
 	ut64 slide = rz_dyldcache_get_slide(cache);
-	ut64 res = va2pa(vaddr, cache->hdr, cache->maps, cache->buf, slide, offset, left);
+	ut64 res = va2pa(vaddr, cache->n_maps, cache->maps, cache->buf, slide, offset, left);
 	if (res == UT64_MAX) {
 		res = 0;
 	}
 	return res;
 }
-
-RZ_API void rz_dyldcache_locsym_entries_by_offset(RzDyldCache *cache, RzList *symbols, SetU *hash, ut64 bin_header_offset) {
-	RzDyldLocSym *locsym = cache->locsym;
-	if (!locsym || !locsym->entries) {
-		return;
-	}
-
-	ut64 i;
-	for (i = 0; i != locsym->entries_count; i++) {
-		cache_locsym_entry_t *entry = &locsym->entries[i];
-		if (entry->dylibOffset != bin_header_offset) {
-			continue;
-		}
-
-		if (entry->nlistStartIndex >= locsym->nlists_count ||
-			entry->nlistStartIndex + entry->nlistCount > locsym->nlists_count) {
-			eprintf("dyldcache: malformed local symbol entry\n");
-			break;
-		}
-
-		ut32 j;
-		for (j = 0; j != entry->nlistCount; j++) {
-			struct MACH0_(nlist) *nlist = &locsym->nlists[j + entry->nlistStartIndex];
-			if (set_u_contains(hash, (ut64)nlist->n_value)) {
-				continue;
-			}
-			set_u_add(hash, (ut64)nlist->n_value);
-			if (nlist->n_strx >= locsym->strings_size) {
-				continue;
-			}
-			char *symstr = &locsym->strings[nlist->n_strx];
-			RzBinSymbol *sym = RZ_NEW0(RzBinSymbol);
-			if (!sym) {
-				return;
-			}
-			sym->type = "LOCAL";
-			sym->vaddr = nlist->n_value;
-			ut64 slide = rz_dyldcache_get_slide(cache);
-			sym->paddr = va2pa(nlist->n_value, cache->hdr, cache->maps, cache->buf, slide, NULL, NULL);
-
-			int len = locsym->strings_size - nlist->n_strx;
-			ut32 k;
-			for (k = 0; k < len; k++) {
-				if (((ut8)symstr[k] & 0xff) == 0xff || !symstr[k]) {
-					len = k;
-					break;
-				}
-			}
-			if (len > 0) {
-				sym->name = rz_str_ndup(symstr, len);
-			} else {
-				sym->name = rz_str_newf("unk_local%d", k);
-			}
-
-			rz_list_append(symbols, sym);
-		}
-		break;
-	}
-}
