@@ -199,7 +199,8 @@ static pyc_object *get_long_object(RzBuffer *buffer) {
 	if (ndigits == 0) {
 		ret->data = strdup("0x0");
 	} else {
-		size = ndigits * 15;
+		// the explicit cast is safe since ndigits is positive
+		size = (size_t)ndigits * 15;
 		size = (size - 1) / 4 + 1;
 		size += 4 + (neg ? 1 : 0);
 		hexstr = malloc(size);
@@ -320,7 +321,6 @@ static pyc_object *get_binary_float_object(RzBuffer *buffer) {
 static pyc_object *get_complex_object(RzBuffer *buffer) {
 	pyc_object *ret = NULL;
 	bool error = false;
-	ut32 size = 0;
 	ut32 n1 = 0;
 	ut32 n2 = 0;
 
@@ -334,7 +334,7 @@ static pyc_object *get_complex_object(RzBuffer *buffer) {
 	} else {
 		n1 = get_st32(buffer, &error);
 	}
-	if (error) {
+	if (error || UT32_ADD_OVFCHK(n1, 1)) {
 		free(ret);
 		return NULL;
 	}
@@ -343,8 +343,7 @@ static pyc_object *get_complex_object(RzBuffer *buffer) {
 		return NULL;
 	}
 	/* object contain string representation of the number */
-	size = rz_buf_read(buffer, s1, n1);
-	if (size != n1) {
+	if (rz_buf_read(buffer, s1, n1) != n1) {
 		RZ_FREE(s1);
 		RZ_FREE(ret);
 		return NULL;
@@ -353,9 +352,10 @@ static pyc_object *get_complex_object(RzBuffer *buffer) {
 
 	if ((magic_int & 0xffff) <= 62061) {
 		n2 = get_ut8(buffer, &error);
-	} else
+	} else {
 		n2 = get_st32(buffer, &error);
-	if (error) {
+	}
+	if (error || UT32_ADD_OVFCHK(n2, 1)) {
 		return NULL;
 	}
 	ut8 *s2 = malloc(n2 + 1);
@@ -363,8 +363,7 @@ static pyc_object *get_complex_object(RzBuffer *buffer) {
 		return NULL;
 	}
 	/* object contain string representation of the number */
-	size = rz_buf_read(buffer, s2, n2);
-	if (size != n2) {
+	if (rz_buf_read(buffer, s2, n2) != n2) {
 		RZ_FREE(s1);
 		RZ_FREE(s2);
 		RZ_FREE(ret);
