@@ -17,25 +17,27 @@ static bool check_buffer(RzBuffer *buff) {
 }
 
 static bool load_buffer(RzBinFile *bf, RzBinObject *obj, RzBuffer *buf, Sdb *sdb) {
-	ut8 MAJOR_MINOR_VERSION;
+	ut8 major_minor_version;
 	LuacBinInfo *bin_info_obj = NULL;
 	LuaProto *proto = NULL;
 	RzBinInfo *general_info = NULL;
 	st32 major;
 	st32 minor;
 
-	rz_buf_read_at(buf, LUAC_VERSION_OFFSET, &MAJOR_MINOR_VERSION, sizeof(MAJOR_MINOR_VERSION)); /* 1-byte in fact */
-	if ((bin_info_obj = RZ_NEW(LuacBinInfo)) == NULL) {
-		return false;
-	}
-	major = (MAJOR_MINOR_VERSION & 0xF0) >> 4;
-	minor = (MAJOR_MINOR_VERSION & 0x0F);
+	rz_buf_read_at(buf, LUAC_VERSION_OFFSET, &major_minor_version, sizeof(major_minor_version)); /* 1-byte in fact */
+	major = (major_minor_version & 0xF0) >> 4;
+	minor = (major_minor_version & 0x0F);
 
 	if (major != 5) {
 		RZ_LOG_ERROR("currently support lua 5.x only\n");
 		return false;
 	}
 
+	bin_info_obj = RZ_NEW(LuacBinInfo);
+	if (!bin_info_obj) {
+		return false;
+	}
+
 	switch (minor) {
 	case 4:
 		proto = lua_parse_body_54(buf, 0x20, bf->size);
@@ -47,6 +49,7 @@ static bool load_buffer(RzBinFile *bf, RzBinObject *obj, RzBuffer *buf, Sdb *sdb
 		break;
 	default:
 		RZ_LOG_ERROR("lua 5.%c not support now\n", minor + '0');
+		free(bin_info_obj);
 		return false;
 	}
 
@@ -54,6 +57,7 @@ static bool load_buffer(RzBinFile *bf, RzBinObject *obj, RzBuffer *buf, Sdb *sdb
 	if (bin_info_obj == NULL) {
 		lua_free_proto_entry(proto);
 		rz_bin_info_free(general_info);
+		free(bin_info_obj);
 		return false;
 	}
 	bin_info_obj->general_info = general_info;
@@ -100,7 +104,7 @@ static RzList *symbols(RzBinFile *bf) {
 		return NULL;
 	}
 
-	return bin_info_obj->symbol_list;
+	return rz_list_clone(bin_info_obj->symbol_list);
 }
 
 static RzList *entries(RzBinFile *bf) {
@@ -112,7 +116,7 @@ static RzList *entries(RzBinFile *bf) {
 		return NULL;
 	}
 
-	return bin_info_obj->entry_list;
+	return rz_list_clone(bin_info_obj->entry_list);
 }
 
 static RzList *strings(RzBinFile *bf) {
@@ -124,7 +128,12 @@ static RzList *strings(RzBinFile *bf) {
 		return NULL;
 	}
 
-	return bin_info_obj->string_list;
+	return rz_list_clone(bin_info_obj->string_list);
+}
+
+static void destroy(RzBinFile *bf) {
+	LuacBinInfo *bin_info_obj = GET_INTERNAL_BIN_INFO_OBJ(bf);
+	luac_build_info_free(bin_info_obj);
 }
 
 RzBinPlugin rz_bin_plugin_luac = {
@@ -133,6 +142,7 @@ RzBinPlugin rz_bin_plugin_luac = {
 	.license = "LGPL3",
 	.get_sdb = NULL,
 	.load_buffer = &load_buffer,
+	.destroy = &destroy,
 	.check_buffer = &check_buffer,
 	.baddr = NULL,
 	.entries = &entries,
