@@ -7,6 +7,7 @@
 #include <rz_debug.h>
 #include <rz_drx.h>
 #include <rz_core.h>
+#include <rz_windows.h>
 #include <signal.h>
 
 RZ_LIB_VERSION(rz_debug);
@@ -102,10 +103,9 @@ static int rz_debug_bp_hit(RzDebug *dbg, RzRegItem *pc_ri, ut64 pc, RzBreakpoint
 		return true;
 	}
 #else
-	int pc_off = dbg->bpsize;
 	/* see if we really have a breakpoint here... */
 	if (!dbg->pc_at_bp_set) {
-		b = rz_bp_get_at(dbg->bp, pc - dbg->bpsize);
+		b = rz_bp_get_ending_at(dbg->bp, pc);
 		if (!b) { /* we don't. nothing left to do */
 			/* Some targets set pc to breakpoint */
 			b = rz_bp_get_at(dbg->bp, pc);
@@ -132,18 +132,21 @@ static int rz_debug_bp_hit(RzDebug *dbg, RzRegItem *pc_ri, ut64 pc, RzBreakpoint
 		eprintf("failed to determine position of pc after breakpoint");
 	}
 
+	int pc_off = 0;
 	if (dbg->pc_at_bp) {
-		pc_off = 0;
 		b = rz_bp_get_at(dbg->bp, pc);
 	} else {
-		b = rz_bp_get_at(dbg->bp, pc - dbg->bpsize);
+		b = rz_bp_get_ending_at(dbg->bp, pc);
+		if (b) {
+			pc_off = b->size;
+		}
 	}
 
 	if (!b) {
 		return true;
 	}
 
-	b = rz_bp_get_at(dbg->bp, pc - dbg->bpsize);
+	b = rz_bp_get_ending_at(dbg->bp, pc);
 	if (!b) { /* we don't. nothing left to do */
 		/* Some targets set pc to breakpoint */
 		b = rz_bp_get_at(dbg->bp, pc);
@@ -264,8 +267,9 @@ static int rz_debug_recoil(RzDebug *dbg, RzDebugRecoilMode rc_mode) {
 }
 
 /* add a breakpoint with some typical values */
-RZ_API RzBreakpointItem *rz_debug_bp_add(RzDebug *dbg, ut64 addr, int hw, bool watch, int rw, char *module, st64 m_delta) {
-	int bpsz = rz_bp_size(dbg->bp);
+RZ_API RZ_BORROW RzBreakpointItem *rz_debug_bp_add(RZ_NONNULL RzDebug *dbg, ut64 addr, int hw, bool watch, int rw, RZ_NULLABLE const char *module, st64 m_delta) {
+	rz_return_val_if_fail(dbg, NULL);
+	int bpsz = rz_bp_size_at(dbg->bp, addr);
 	RzBreakpointItem *bpi;
 	const char *module_name = module;
 	RzListIter *iter;
@@ -286,7 +290,7 @@ RZ_API RzBreakpointItem *rz_debug_bp_add(RzDebug *dbg, ut64 addr, int hw, bool w
 			}
 			rz_list_free(list);
 		} else {
-			//module holds the address
+			// module holds the address
 			addr = (ut64)rz_num_math(dbg->num, module);
 			if (!addr) {
 				return NULL;
@@ -302,7 +306,7 @@ RZ_API RzBreakpointItem *rz_debug_bp_add(RzDebug *dbg, ut64 addr, int hw, bool w
 					m_delta = addr - map->addr;
 				}
 				perm = ((map->perm & 1) << 2) | (map->perm & 2) | ((map->perm & 4) >> 2);
-				if (!(perm & RZ_BP_PROT_EXEC)) {
+				if (!(perm & RZ_PERM_X)) {
 					eprintf("WARNING: setting bp within mapped memory without exec perm\n");
 				}
 				break;
@@ -314,7 +318,7 @@ RZ_API RzBreakpointItem *rz_debug_bp_add(RzDebug *dbg, ut64 addr, int hw, bool w
 		}
 	}
 	if (!module) {
-		//express db breakpoints as dbm due to ASLR when saving into project
+		// express db breakpoints as dbm due to ASLR when saving into project
 		rz_debug_map_sync(dbg);
 		rz_list_foreach (dbg->maps, iter, map) {
 			if (addr >= map->addr && addr < map->addr_end) {
@@ -325,12 +329,12 @@ RZ_API RzBreakpointItem *rz_debug_bp_add(RzDebug *dbg, ut64 addr, int hw, bool w
 		}
 	}
 	if (watch) {
-		hw = 1; //XXX
+		hw = 1; // XXX
 		bpi = rz_bp_watch_add(dbg->bp, addr, bpsz, hw, rw);
 	} else {
 		bpi = hw
-			? rz_bp_add_hw(dbg->bp, addr, bpsz, RZ_BP_PROT_EXEC)
-			: rz_bp_add_sw(dbg->bp, addr, bpsz, RZ_BP_PROT_EXEC);
+			? rz_bp_add_hw(dbg->bp, addr, bpsz, RZ_PERM_X)
+			: rz_bp_add_sw(dbg->bp, addr, bpsz, RZ_PERM_X);
 	}
 	if (bpi) {
 		if (module_name) {
@@ -352,7 +356,8 @@ void free_tracenodes_kv(HtUPKv *kv) {
 	free(kv->value);
 }
 
-RZ_API RzDebug *rz_debug_new(int hard) {
+RZ_API RZ_OWN RzDebug *rz_debug_new(RZ_BORROW RZ_NONNULL RzBreakpointContext *bp_ctx) {
+	rz_return_val_if_fail(bp_ctx, NULL);
 	RzDebug *dbg = RZ_NEW0(RzDebug);
 	if (!dbg) {
 		return NULL;
@@ -373,7 +378,6 @@ RZ_API RzDebug *rz_debug_new(int hard) {
 	dbg->trace_execs = 0;
 	dbg->analysis = NULL;
 	dbg->pid = -1;
-	dbg->bpsize = 1;
 	dbg->tid = -1;
 	dbg->tree = rz_tree_new();
 	dbg->tracenodes = ht_up_new(NULL, free_tracenodes_kv, NULL);
@@ -395,12 +399,11 @@ RZ_API RzDebug *rz_debug_new(int hard) {
 	dbg->main_arena_resolved = false;
 	dbg->glibc_version = 231; /* default version ubuntu 20 */
 	rz_debug_signal_init(dbg);
-	if (hard) {
-		dbg->bp = rz_bp_new();
-		rz_debug_plugin_init(dbg);
-		dbg->bp->iob.init = false;
-		dbg->bp->baddr = 0;
-	}
+	dbg->bp = rz_bp_new(bp_ctx);
+	rz_debug_plugin_init(dbg);
+	dbg->bp->iob.init = false;
+	dbg->bp->baddr = 0;
+	dbg->nt_x86_xstate_supported = true;
 	return dbg;
 }
 
@@ -411,9 +414,7 @@ RZ_API void rz_debug_tracenodes_reset(RzDebug *dbg) {
 
 RZ_API RzDebug *rz_debug_free(RzDebug *dbg) {
 	if (dbg) {
-		// TODO: free it correctly.. we must ensure this is an instance and not a reference..
 		rz_bp_free(dbg->bp);
-		//rz_reg_free(&dbg->reg);
 		free(dbg->snap_path);
 		rz_list_free(dbg->maps);
 		rz_list_free(dbg->maps_user);
@@ -430,6 +431,7 @@ RZ_API RzDebug *rz_debug_free(RzDebug *dbg) {
 		rz_analysis_op_free(dbg->cur_op);
 		dbg->trace = NULL;
 		rz_egg_free(dbg->egg);
+		rz_reg_free(dbg->reg);
 		free(dbg->arch);
 		free(dbg->glob_libs);
 		free(dbg->glob_unlibs);
@@ -444,7 +446,7 @@ RZ_API int rz_debug_attach(RzDebug *dbg, int pid) {
 		ret = dbg->cur->attach(dbg, pid);
 		if (ret != -1) {
 			dbg->reason.type = RZ_DEBUG_REASON_NONE; // after a successful attach, the process is not dead
-			rz_debug_select(dbg, pid, ret); //dbg->pid, dbg->tid);
+			rz_debug_select(dbg, pid, ret); // dbg->pid, dbg->tid);
 		}
 	}
 	return ret;
@@ -461,6 +463,11 @@ RZ_API int rz_debug_stop(RzDebug *dbg) {
 RZ_API bool rz_debug_set_arch(RzDebug *dbg, const char *arch, int bits) {
 	if (arch && dbg && dbg->cur) {
 		switch (bits) {
+		case 16:
+			if (dbg->cur->bits & RZ_SYS_BITS_16) {
+				dbg->bits = RZ_SYS_BITS_16;
+			}
+			break;
 		case 27:
 			if (dbg->cur->bits == 27) {
 				dbg->bits = 27;
@@ -511,7 +518,7 @@ RZ_API ut64 rz_debug_execute(RzDebug *dbg, const ut8 *buf, int len, int restore)
 	risp = rz_reg_get(dbg->reg, dbg->reg->name[RZ_REG_NAME_SP], RZ_REG_TYPE_GPR);
 	if (ripc) {
 		rz_debug_reg_sync(dbg, RZ_REG_TYPE_GPR, false);
-		orig = rz_reg_get_bytes(dbg->reg, RZ_REG_TYPE_ALL, &orig_sz);
+		orig = rz_reg_get_bytes(dbg->reg, RZ_REG_TYPE_ANY, &orig_sz);
 		if (!orig) {
 			eprintf("Cannot get register arena bytes\n");
 			return 0LL;
@@ -527,13 +534,13 @@ RZ_API ut64 rz_debug_execute(RzDebug *dbg, const ut8 *buf, int len, int restore)
 		dbg->iob.read_at(dbg->iob.io, rpc, backup, len);
 		dbg->iob.read_at(dbg->iob.io, rsp, stackbackup, len);
 
-		rz_bp_add_sw(dbg->bp, rpc + len, dbg->bpsize, RZ_BP_PROT_EXEC);
+		rz_bp_add_sw(dbg->bp, rpc + len, 0, RZ_PERM_X);
 
 		/* execute code here */
 		dbg->iob.write_at(dbg->iob.io, rpc, buf, len);
-		//rz_bp_add_sw (dbg->bp, rpc+len, 4, RZ_BP_PROT_EXEC);
+		// rz_bp_add_sw (dbg->bp, rpc+len, 4, RZ_PERM_X);
 		rz_debug_continue(dbg);
-		//rz_bp_del (dbg->bp, rpc+len);
+		// rz_bp_del (dbg->bp, rpc+len);
 		/* TODO: check if stopped in breakpoint or not */
 
 		rz_bp_del(dbg->bp, rpc + len);
@@ -618,7 +625,7 @@ RZ_API bool rz_debug_select(RzDebug *dbg, int pid, int tid) {
 		RzCore *core = (RzCore *)dbg->corebind.core;
 
 		rz_reg_arena_swap(core->dbg->reg, true);
-		rz_debug_reg_sync(dbg, RZ_REG_TYPE_ALL, false);
+		rz_debug_reg_sync(dbg, RZ_REG_TYPE_ANY, false);
 
 		pc = rz_debug_reg_get(dbg, "PC");
 		core->offset = pc;
@@ -698,7 +705,7 @@ RZ_API RzDebugReasonType rz_debug_wait(RzDebug *dbg, RzBreakpointItem **bp) {
 			};
 			rz_event_send(dbg->ev, RZ_EVENT_DEBUG_PROCESS_FINISHED, &event);
 			// XXX(jjd): TODO: handle fallback or something else
-			//rz_debug_select (dbg, -1, -1);
+			// rz_debug_select (dbg, -1, -1);
 			return RZ_DEBUG_REASON_DEAD;
 		}
 
@@ -798,6 +805,9 @@ RZ_API int rz_debug_step_soft(RzDebug *dbg) {
 		return false;
 	}
 
+	const bool has_lr_reg = rz_reg_get_name(dbg->reg, RZ_REG_NAME_LR);
+	const bool arch_ret_is_pop = !strcmp(dbg->arch, "arm") && dbg->bits <= RZ_SYS_BITS_32;
+
 	pc = rz_debug_reg_get(dbg, dbg->reg->name[RZ_REG_NAME_PC]);
 	sp = rz_debug_reg_get(dbg, dbg->reg->name[RZ_REG_NAME_SP]);
 
@@ -815,8 +825,15 @@ RZ_API int rz_debug_step_soft(RzDebug *dbg) {
 	}
 	switch (op.type) {
 	case RZ_ANALYSIS_OP_TYPE_RET:
-		dbg->iob.read_at(dbg->iob.io, sp, (ut8 *)&sp_top, 8);
-		next[0] = (dbg->bits == RZ_SYS_BITS_32) ? sp_top.r32[0] : sp_top.r64;
+		if (arch_ret_is_pop && op.stackop == RZ_ANALYSIS_STACK_INC) {
+			dbg->iob.read_at(dbg->iob.io, sp - op.stackptr - 4, (ut8 *)&sp_top, 4);
+			next[0] = sp_top.r32[0];
+		} else if (has_lr_reg) {
+			next[0] = rz_debug_reg_get(dbg, dbg->reg->name[RZ_REG_NAME_LR]);
+		} else {
+			dbg->iob.read_at(dbg->iob.io, sp, (ut8 *)&sp_top, 8);
+			next[0] = (dbg->bits <= RZ_SYS_BITS_32) ? sp_top.r32[0] : sp_top.r64;
+		}
 		br = 1;
 		break;
 	case RZ_ANALYSIS_OP_TYPE_CJMP:
@@ -842,7 +859,7 @@ RZ_API int rz_debug_step_soft(RzDebug *dbg) {
 		if (!dbg->iob.read_at(dbg->iob.io, r, (ut8 *)&memval, 8)) {
 			next[0] = op.addr + op.size;
 		} else {
-			next[0] = (dbg->bits == RZ_SYS_BITS_32) ? memval.r32[0] : memval.r64;
+			next[0] = (dbg->bits <= RZ_SYS_BITS_32) ? memval.r32[0] : memval.r64;
 		}
 		br = 1;
 		break;
@@ -857,7 +874,7 @@ RZ_API int rz_debug_step_soft(RzDebug *dbg) {
 		if (!dbg->iob.read_at(dbg->iob.io, r * op.scale + op.disp, (ut8 *)&memval, 8)) {
 			next[0] = op.addr + op.size;
 		} else {
-			next[0] = (dbg->bits == RZ_SYS_BITS_32) ? memval.r32[0] : memval.r64;
+			next[0] = (dbg->bits <= RZ_SYS_BITS_32) ? memval.r32[0] : memval.r64;
 		}
 		br = 1;
 		break;
@@ -867,8 +884,12 @@ RZ_API int rz_debug_step_soft(RzDebug *dbg) {
 		break;
 	}
 
+	const int align = rz_analysis_archinfo(dbg->analysis, RZ_ANALYSIS_ARCHINFO_ALIGN);
 	for (i = 0; i < br; i++) {
-		RzBreakpointItem *bpi = rz_bp_add_sw(dbg->bp, next[i], dbg->bpsize, RZ_BP_PROT_EXEC);
+		if (align > 1) {
+			next[i] = next[i] - (next[i] % align);
+		}
+		RzBreakpointItem *bpi = rz_bp_add_sw(dbg->bp, next[i], 0, RZ_PERM_X);
 		if (bpi) {
 			bpi->swstep = true;
 		}
@@ -1071,7 +1092,7 @@ RZ_API int rz_debug_step_over(RzDebug *dbg, int steps) {
 				return steps_taken;
 			}
 		} else if ((op.prefix & (RZ_ANALYSIS_OP_PREFIX_REP | RZ_ANALYSIS_OP_PREFIX_REPNE | RZ_ANALYSIS_OP_PREFIX_LOCK))) {
-			//eprintf ("REP: skip to next instruction...\n");
+			// eprintf ("REP: skip to next instruction...\n");
 			if (!rz_debug_continue_until(dbg, ins_size)) {
 				eprintf("step over failed over rep\n");
 				return steps_taken;
@@ -1135,7 +1156,7 @@ RZ_API int rz_debug_continue_kill(RzDebug *dbg, int sig) {
 			if (reg->cnum <= dbg->session->cnum) {
 				continue;
 			}
-			has_bp = rz_bp_get_in(dbg->bp, reg->data, RZ_BP_PROT_EXEC) != NULL;
+			has_bp = rz_bp_get_in(dbg->bp, reg->data, RZ_PERM_X) != NULL;
 			if (has_bp) {
 				eprintf("hit breakpoint at: 0x%" PFMT64x " cnum: %d\n", reg->data, reg->cnum);
 				rz_debug_goto_cnum(dbg, reg->cnum);
@@ -1169,7 +1190,7 @@ repeat:
 		}
 		/* tell the inferior to go! */
 		ret = dbg->cur->cont(dbg, dbg->pid, dbg->tid, sig);
-		//XXX(jjd): why? //dbg->reason.signum = 0;
+		// XXX(jjd): why? //dbg->reason.signum = 0;
 		reason = rz_debug_wait(dbg, &bp);
 	} else {
 		return 0;
@@ -1307,14 +1328,12 @@ repeat:
 }
 
 RZ_API int rz_debug_continue(RzDebug *dbg) {
-	return rz_debug_continue_kill(dbg, 0); //dbg->reason.signum);
+	return rz_debug_continue_kill(dbg, 0); // dbg->reason.signum);
 }
 
-#if __WINDOWS__
 RZ_API int rz_debug_continue_pass_exception(RzDebug *dbg) {
-	return rz_debug_continue_kill(dbg, DBG_EXCEPTION_NOT_HANDLED);
+	return rz_debug_continue_kill(dbg, dbg->reason.signum);
 }
-#endif
 
 RZ_API int rz_debug_continue_until_nontraced(RzDebug *dbg) {
 	eprintf("TODO\n");
@@ -1383,9 +1402,9 @@ static int rz_debug_continue_until_internal(RzDebug *dbg, ut64 addr, bool block)
 		return false;
 	}
 	// Check if there was another breakpoint set at addr
-	bool has_bp = rz_bp_get_in(dbg->bp, addr, RZ_BP_PROT_EXEC) != NULL;
+	bool has_bp = rz_bp_get_in(dbg->bp, addr, RZ_PERM_X) != NULL;
 	if (!has_bp) {
-		rz_bp_add_sw(dbg->bp, addr, dbg->bpsize, RZ_BP_PROT_EXEC);
+		rz_bp_add_sw(dbg->bp, addr, 0, RZ_PERM_X);
 	}
 
 	// Continue until the bp is reached
@@ -1433,7 +1452,7 @@ RZ_API bool rz_debug_continue_back(RzDebug *dbg) {
 		if (reg->cnum >= dbg->session->cnum) {
 			continue;
 		}
-		has_bp = rz_bp_get_in(dbg->bp, reg->data, RZ_BP_PROT_EXEC) != NULL;
+		has_bp = rz_bp_get_in(dbg->bp, reg->data, RZ_PERM_X) != NULL;
 		if (has_bp) {
 			cnum = reg->cnum;
 			eprintf("hit breakpoint at: 0x%" PFMT64x " cnum: %d\n", reg->data, reg->cnum);
@@ -1484,7 +1503,7 @@ static int show_syscall(RzDebug *dbg, const char *sysreg) {
 }
 
 RZ_API int rz_debug_continue_syscalls(RzDebug *dbg, int *sc, int n_sc) {
-	int i, err, reg, ret = false;
+	int i, reg, ret = false;
 	if (!dbg || !dbg->cur || rz_debug_is_dead(dbg)) {
 		return false;
 	}
@@ -1498,8 +1517,8 @@ RZ_API int rz_debug_continue_syscalls(RzDebug *dbg, int *sc, int n_sc) {
 		eprintf("--> cannot read registers\n");
 		return -1;
 	}
-	rz_debug_reg_get_err(dbg, "SN", &err, NULL);
-	if (err) {
+
+	if (!rz_reg_get_by_role(dbg->reg, RZ_REG_NAME_SN)) {
 		eprintf("Cannot find 'sn' register for current arch-os.\n");
 		return -1;
 	}
@@ -1569,6 +1588,11 @@ RZ_API int rz_debug_syscall(RzDebug *dbg, int num) {
 	return (int)ret;
 }
 
+/// check whether rz_debug_kill() will not definitely fail (for example because kill is unimplemented by the plugin)
+RZ_API bool rz_debug_can_kill(RzDebug *dbg) {
+	return !rz_debug_is_dead(dbg) && dbg->cur && dbg->cur->kill;
+}
+
 RZ_API int rz_debug_kill(RzDebug *dbg, int pid, int tid, int sig) {
 	if (rz_debug_is_dead(dbg)) {
 		return false;
@@ -1592,14 +1616,14 @@ RZ_API RzList *rz_debug_frames(RzDebug *dbg, ut64 at) {
 
 /* TODO: Implement fork and clone */
 RZ_API int rz_debug_child_fork(RzDebug *dbg) {
-	//if (dbg && dbg->cur && dbg->cur->frames)
-	//return dbg->cur->frames (dbg);
+	// if (dbg && dbg->cur && dbg->cur->frames)
+	// return dbg->cur->frames (dbg);
 	return 0;
 }
 
 RZ_API int rz_debug_child_clone(RzDebug *dbg) {
-	//if (dbg && dbg->cur && dbg->cur->frames)
-	//return dbg->cur->frames (dbg);
+	// if (dbg && dbg->cur && dbg->cur->frames)
+	// return dbg->cur->frames (dbg);
 	return 0;
 }
 
@@ -1657,7 +1681,7 @@ RZ_API ut64 rz_debug_get_baddr(RzDebug *dbg, const char *file) {
 	if (!dbg || !dbg->iob.io || !dbg->iob.io->desc) {
 		return 0LL;
 	}
-	if (!strcmp(dbg->iob.io->desc->plugin->name, "gdb")) { //this is very bad
+	if (!strcmp(dbg->iob.io->desc->plugin->name, "gdb")) { // this is very bad
 		// Tell gdb that we want baddr, not full mem map
 		dbg->iob.system(dbg->iob.io, "baddr");
 	}
