@@ -6,17 +6,21 @@
 
 static char *rz_debug_native_reg_profile(RzDebug *dbg) {
 #if __WINDOWS__
-	/*_______
+/*_______
  |   |   |
  |___|___|
  |   |   |
  |___|___|
 */
-	if (dbg->bits & RZ_SYS_BITS_64) {
+#if defined(__arm64__)
+#include "reg/windows-arm64.h"
+#elif defined(__arm__)
+#include "reg/windows-arm.h"
+#elif defined(__x86_64__)
 #include "reg/windows-x64.h"
-	} else {
+#elif defined(__i386__)
 #include "reg/windows-x86.h"
-	}
+#endif
 #elif (__OpenBSD__ || __NetBSD__)
 /*                           __.--..__
        \-/-/-/    _ __  _.--'  _.--'
