@@ -15,9 +15,7 @@
 #include <sys/types.h>
 
 #if __APPLE__ && HAVE_FORK
-#if !__POWERPC__
 #include <spawn.h>
-#endif
 #include <sys/wait.h>
 #include <mach/exception_types.h>
 #include <mach/mach_init.h>
@@ -288,7 +286,7 @@ static void setASLR(RzRunProfile *r, int enabled) {
 #endif
 }
 
-#if __APPLE__ && !__POWERPC__
+#if __APPLE__
 #else
 #if HAVE_OPENPTY && HAVE_FORKPTY && HAVE_LOGIN_TTY
 static void restore_saved_fd(int saved, bool restore, int fd) {
@@ -389,9 +387,9 @@ static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err)
 #endif
 
 static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
-#if __APPLE__ && !__POWERPC__
-	//XXX handle this in other layer since things changes a little bit
-	//this seems like a really good place to refactor stuff
+#if __APPLE__
+	// XXX handle this in other layer since things changes a little bit
+	// this seems like a really good place to refactor stuff
 	return 0;
 #else
 	if (!cmd || !*cmd) {
@@ -1018,7 +1016,9 @@ RZ_API int rz_run_config_env(RzRunProfile *p) {
 		if (p->_preload) {
 			eprintf("WARNING: Only one library can be opened at a time\n");
 		}
-		p->_preload = rz_str_rz_prefix(RZ_JOIN_2_PATHS(RZ_LIBDIR, "librz." RZ_LIB_EXT));
+		char *libdir = rz_path_libdir();
+		p->_preload = rz_file_path_join(libdir, "librz." RZ_LIB_EXT);
+		free(libdir);
 	}
 	if (p->_libpath) {
 #if __WINDOWS__
@@ -1081,14 +1081,14 @@ RZ_API int rz_run_start(RzRunProfile *p) {
 		exit(rz_sys_execv(p->_program, (char *const *)p->_args));
 	}
 #endif
-#if __APPLE__ && !__POWERPC__ && HAVE_FORK
+#if __APPLE__ && HAVE_FORK
 	posix_spawnattr_t attr = { 0 };
 	pid_t pid = -1;
 	int ret;
 	posix_spawnattr_init(&attr);
 	if (p->_args[0]) {
 		char **envp = rz_sys_get_environ();
-		ut32 spflags = 0; //POSIX_SPAWN_START_SUSPENDED;
+		ut32 spflags = 0; // POSIX_SPAWN_START_SUSPENDED;
 		spflags |= POSIX_SPAWN_SETEXEC;
 		if (p->_aslr == 0) {
 #define _POSIX_SPAWN_DISABLE_ASLR 0x0100
@@ -1114,15 +1114,8 @@ RZ_API int rz_run_start(RzRunProfile *p) {
 		switch (ret) {
 		case 0:
 			break;
-		case 22:
-			eprintf("posix_spawnp: Invalid argument\n");
-			break;
-		case 86:
-			eprintf("posix_spawnp: Unsupported architecture\n");
-			break;
 		default:
-			eprintf("posix_spawnp: unknown error %d\n", ret);
-			perror("posix_spawnp");
+			eprintf("posix_spawnp: %s\n", strerror(ret));
 			break;
 		}
 		exit(ret);
@@ -1340,7 +1333,10 @@ RZ_API char *rz_run_get_environ_profile(char **env) {
 		char *v = strchr(k, '=');
 		if (v) {
 			*v++ = 0;
-			v = rz_str_escape_latin1(v, false, true, true);
+			RzStrEscOptions opt = { 0 };
+			opt.show_asciidot = false;
+			opt.esc_bslash = true;
+			v = rz_str_escape_8bit(v, true, &opt);
 			if (v) {
 				rz_strbuf_appendf(sb, "setenv=%s=\"%s\"\n", k, v);
 				free(v);
