@@ -148,6 +148,21 @@ RZ_API void rz_type_db_set_bits(RzTypeDB *typedb, int bits) {
 	set_default_type(typedb->target, bits);
 }
 
+/**
+ * \brief Set the RzType target adress size
+ *
+ * Important for calculating some types size, especially
+ * pointers's size.
+ *
+ * \param typedb RzTypeDB instance
+ * \param bits size of an address in bits. If <= 0, then
+ *        the value from rz_type_db_set_bits() is used.
+ */
+RZ_API void rz_type_db_set_address_bits(RzTypeDB *typedb, int addr_bits) {
+	rz_return_if_fail(typedb);
+	typedb->target->addr_bits = addr_bits;
+}
+
 /**
  * \brief Set the RzType target architecture operating system
  *
@@ -194,9 +209,7 @@ RZ_API void rz_type_db_set_endian(RzTypeDB *typedb, bool big_endian) {
  * \param typedb RzTypeDB instance
  */
 RZ_API ut8 rz_type_db_pointer_size(const RzTypeDB *typedb) {
-	// TODO: Handle more special cases where the pointer
-	// size is different from the target bitness
-	return typedb->target->bits;
+	return typedb->target->addr_bits > 0 ? typedb->target->addr_bits : typedb->target->bits;
 }
 
 /**
@@ -232,12 +245,12 @@ RZ_API bool rz_type_db_del(RzTypeDB *typedb, RZ_NONNULL const char *name) {
  * a different layout, depending on the operating system or bitness.
  *
  * \param typedb Types Database instance
- * \param dir_prefix Directory where all type libraries are installed
+ * \param types_dir Directory where all type libraries are installed
  * \param arch Architecture of the analysis session
  * \param bits Bitness of the analysis session
  * \param os Operating system of the analysis session
  */
-RZ_API void rz_type_db_init(RzTypeDB *typedb, const char *dir_prefix, const char *arch, int bits, const char *os) {
+RZ_API void rz_type_db_init(RzTypeDB *typedb, const char *types_dir, const char *arch, int bits, const char *os) {
 	rz_return_if_fail(typedb && typedb->types && typedb->formats);
 
 	// A workaround to fix loading incorrectly detected MacOS binaries
@@ -247,15 +260,17 @@ RZ_API void rz_type_db_init(RzTypeDB *typedb, const char *dir_prefix, const char
 
 	// At first we load the basic types
 	// Atomic types
-	const char *dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-atomic.sdb"), dir_prefix);
+	char *dbpath = rz_file_path_join(types_dir, "types-atomic.sdb");
 	if (rz_type_db_load_sdb(typedb, dbpath)) {
 		RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 	}
+	free(dbpath);
 	// C runtime types
-	dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-libc.sdb"), dir_prefix);
+	dbpath = rz_file_path_join(types_dir, "types-libc.sdb");
 	if (rz_type_db_load_sdb(typedb, dbpath)) {
 		RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 	}
+	free(dbpath);
 
 	// We do not load further if bits are not specified
 	if (bits <= 0) {
@@ -263,11 +278,12 @@ RZ_API void rz_type_db_init(RzTypeDB *typedb, const char *dir_prefix, const char
 	}
 
 	// Bits-specific types that are independent from architecture or OS
-	dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%d.sdb"),
-		dir_prefix, bits);
+	char tmp[100];
+	dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%d.sdb", bits));
 	if (rz_type_db_load_sdb(typedb, dbpath)) {
 		RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 	}
+	free(dbpath);
 
 	// We do not load further if architecture is not specified
 	if (!arch) {
@@ -275,56 +291,57 @@ RZ_API void rz_type_db_init(RzTypeDB *typedb, const char *dir_prefix, const char
 	}
 
 	// Architecture-specific types
-	dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%s.sdb"),
-		dir_prefix, arch);
+	dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%s.sdb", arch));
 	if (rz_type_db_load_sdb(typedb, dbpath)) {
 		RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 	}
+	free(dbpath);
 
 	// Architecture- and bits-specific types
-	dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%s-%d.sdb"),
-		dir_prefix, arch, bits);
+	dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%s-%d.sdb", arch, bits));
 	if (rz_type_db_load_sdb(typedb, dbpath)) {
 		RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 	}
+	free(dbpath);
 
 	if (os) {
 		// OS-specific types
-		dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%s.sdb"),
-			dir_prefix, os);
+		dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%s.sdb", os));
 		if (rz_type_db_load_sdb(typedb, dbpath)) {
 			RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 		}
-		dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%s-%d.sdb"),
-			dir_prefix, os, bits);
+		free(dbpath);
+		dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%s-%d.sdb", os, bits));
 		if (rz_type_db_load_sdb(typedb, dbpath)) {
 			RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 		}
-		dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%s-%s.sdb"),
-			dir_prefix, arch, os);
+		free(dbpath);
+		dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%s-%s.sdb", arch, os));
 		if (rz_type_db_load_sdb(typedb, dbpath)) {
 			RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 		}
-		dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "types-%s-%s-%d.sdb"),
-			dir_prefix, arch, os, bits);
+		free(dbpath);
+		dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "types-%s-%s-%d.sdb", arch, os, bits));
 		if (rz_type_db_load_sdb(typedb, dbpath)) {
 			RZ_LOG_DEBUG("types: loaded \"%s\"\n", dbpath);
 		}
+		free(dbpath);
 	}
 
 	// Then, after all basic types are initialized, we load function types
 	// that use loaded previously base types for return and arguments
-	dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "functions-libc.sdb"), dir_prefix);
+	dbpath = rz_file_path_join(types_dir, "functions-libc.sdb");
 	if (rz_type_db_load_callables_sdb(typedb, dbpath)) {
 		RZ_LOG_DEBUG("callable types: loaded \"%s\"\n", dbpath);
 	}
+	free(dbpath);
 	// OS-specific function types
 	if (os) {
-		dbpath = sdb_fmt(RZ_JOIN_3_PATHS("%s", RZ_SDB_TYPES, "functions-%s.sdb"),
-			dir_prefix, os);
+		dbpath = rz_file_path_join(types_dir, rz_strf(tmp, "functions-%s.sdb", os));
 		if (rz_type_db_load_callables_sdb(typedb, dbpath)) {
 			RZ_LOG_DEBUG("callable types: loaded \"%s\"\n", dbpath);
 		}
+		free(dbpath);
 	}
 }
 
@@ -335,11 +352,11 @@ RZ_API void rz_type_db_init(RzTypeDB *typedb, const char *dir_prefix, const char
  * for base types and function types.
  *
  * \param typedb Types Database instance
- * \param dir_prefix Directory where all type libraries are installed
+ * \param types_dir Directory where all type libraries are installed
  */
-RZ_API void rz_type_db_reload(RzTypeDB *typedb, const char *dir_prefix) {
+RZ_API void rz_type_db_reload(RzTypeDB *typedb, const char *types_dir) {
 	rz_type_db_purge(typedb);
-	rz_type_db_init(typedb, dir_prefix, typedb->target->cpu, typedb->target->bits, typedb->target->os);
+	rz_type_db_init(typedb, types_dir, typedb->target->cpu, typedb->target->bits, typedb->target->os);
 }
 
 // Listing all available types by category
@@ -488,7 +505,7 @@ RZ_API RzBaseType *rz_type_db_get_enum(const RzTypeDB *typedb, RZ_NONNULL const
  * \param name The name of the enum to search in
  * \param val The value to search for
  */
-RZ_API RZ_BORROW char *rz_type_db_enum_member_by_val(const RzTypeDB *typedb, RZ_NONNULL const char *name, ut64 val) {
+RZ_API RZ_BORROW const char *rz_type_db_enum_member_by_val(const RzTypeDB *typedb, RZ_NONNULL const char *name, ut64 val) {
 	rz_return_val_if_fail(typedb && name, NULL);
 	RzBaseType *btype = rz_type_db_get_base_type(typedb, name);
 	if (!btype) {
@@ -791,225 +808,6 @@ RZ_API ut64 rz_type_db_get_bitsize(const RzTypeDB *typedb, RZ_NONNULL RzType *ty
 	return 0;
 }
 
-struct HelperBufs {
-	RzStrBuf *arraybuf;
-	RzStrBuf *ptrbuf;
-};
-
-static void helper_bufs_init(struct HelperBufs *hbs) {
-	hbs->arraybuf = rz_strbuf_new("");
-	hbs->ptrbuf = rz_strbuf_new("");
-}
-
-static void helper_bufs_fini(struct HelperBufs *hbs) {
-	rz_strbuf_free(hbs->arraybuf);
-	rz_strbuf_free(hbs->ptrbuf);
-}
-
-static char *type_as_string(const RzTypeDB *typedb, RZ_NONNULL const RzType *type, RZ_NONNULL struct HelperBufs *bufs) {
-	rz_return_val_if_fail(typedb && type && bufs, NULL);
-
-	RzStrBuf *buf = rz_strbuf_new("");
-	switch (type->kind) {
-	case RZ_TYPE_KIND_IDENTIFIER: {
-		// Here it can be any of the RzBaseType
-		RzBaseType *btype = rz_type_db_get_base_type(typedb, type->identifier.name);
-		if (!btype) {
-			rz_strbuf_append(buf, "unknown_t");
-		} else {
-			if (type->identifier.is_const) {
-				rz_strbuf_append(buf, "const ");
-			}
-			switch (btype->kind) {
-			case RZ_BASE_TYPE_KIND_UNION:
-				rz_strbuf_append(buf, "union ");
-				break;
-			case RZ_BASE_TYPE_KIND_STRUCT:
-				rz_strbuf_append(buf, "struct ");
-				break;
-			default:
-				break;
-			}
-			rz_strbuf_append(buf, btype->name);
-		}
-		if (!rz_strbuf_is_empty(bufs->ptrbuf) || !rz_strbuf_is_empty(bufs->arraybuf)) {
-			rz_strbuf_appendf(buf, " %s%s", rz_strbuf_get(bufs->ptrbuf), rz_strbuf_get(bufs->arraybuf));
-		}
-		break;
-	}
-	case RZ_TYPE_KIND_POINTER: {
-		// A pointer to the function is a special case
-		if (rz_type_is_callable_ptr_nested(type)) {
-			char *typestr = rz_type_callable_ptr_as_string(typedb, type);
-			rz_strbuf_append(buf, typestr);
-			free(typestr);
-		} else {
-			if (type->pointer.is_const) {
-				rz_strbuf_prepend(bufs->ptrbuf, "* const ");
-			} else {
-				rz_strbuf_prepend(bufs->ptrbuf, "*");
-			}
-			char *typestr = type_as_string(typedb, type->pointer.type, bufs);
-			rz_strbuf_append(buf, typestr);
-			free(typestr);
-		}
-		break;
-	}
-	case RZ_TYPE_KIND_ARRAY: {
-		rz_strbuf_appendf(bufs->arraybuf, "[%" PFMT64d "]", type->array.count);
-		char *typestr = type_as_string(typedb, type->array.type, bufs);
-		rz_strbuf_append(buf, typestr);
-		free(typestr);
-		break;
-	}
-	case RZ_TYPE_KIND_CALLABLE: {
-		char *callstr = rz_type_callable_as_string(typedb, type->callable);
-		rz_strbuf_append(buf, callstr);
-		free(callstr);
-		break;
-	}
-	}
-	return rz_strbuf_drain(buf);
-}
-
-static char *type_as_string_decl(const RzTypeDB *typedb, RZ_NONNULL const RzType *type, RZ_NONNULL struct HelperBufs *bufs) {
-	rz_return_val_if_fail(typedb && type && bufs, NULL);
-
-	RzStrBuf *buf = rz_strbuf_new("");
-	switch (type->kind) {
-	case RZ_TYPE_KIND_IDENTIFIER: {
-		rz_return_val_if_fail(type->identifier.name, NULL);
-		// Here it can be any of the RzBaseType
-		RzBaseType *btype = rz_type_db_get_base_type(typedb, type->identifier.name);
-		if (!btype) {
-			rz_strbuf_append(buf, "unknown_t");
-		} else {
-			char *btypestr = btype->kind == RZ_BASE_TYPE_KIND_TYPEDEF ? strdup(btype->name) : rz_type_db_base_type_as_string(typedb, btype);
-			if (type->identifier.is_const) {
-				rz_strbuf_appendf(buf, "const %s", btypestr);
-			} else {
-				rz_strbuf_append(buf, btypestr);
-			}
-			free(btypestr);
-		}
-		if (!rz_strbuf_is_empty(bufs->ptrbuf) || !rz_strbuf_is_empty(bufs->arraybuf)) {
-			rz_strbuf_appendf(buf, " %s%s", rz_strbuf_get(bufs->ptrbuf), rz_strbuf_get(bufs->arraybuf));
-		}
-		break;
-	}
-	case RZ_TYPE_KIND_POINTER: {
-		// A pointer to the function is a special case
-		if (rz_type_is_callable_ptr_nested(type)) {
-			char *typestr = rz_type_callable_ptr_as_string(typedb, type);
-			rz_strbuf_append(buf, typestr);
-			free(typestr);
-		} else {
-			if (type->pointer.is_const) {
-				rz_strbuf_prepend(bufs->ptrbuf, "* const ");
-			} else {
-				rz_strbuf_prepend(bufs->ptrbuf, "*");
-			}
-			char *typestr = type_as_string_decl(typedb, type->pointer.type, bufs);
-			rz_strbuf_append(buf, typestr);
-			free(typestr);
-		}
-		break;
-	}
-	case RZ_TYPE_KIND_ARRAY: {
-		rz_strbuf_appendf(bufs->arraybuf, "[%" PFMT64d "]", type->array.count);
-		char *typestr = type_as_string_decl(typedb, type->array.type, bufs);
-		rz_strbuf_append(buf, typestr);
-		free(typestr);
-		break;
-	}
-	case RZ_TYPE_KIND_CALLABLE: {
-		char *callstr = rz_type_callable_as_string(typedb, type->callable);
-		rz_strbuf_append(buf, callstr);
-		free(callstr);
-		break;
-	}
-	}
-	char *result = rz_strbuf_drain(buf);
-	return result;
-}
-
-static char *type_as_string_identifier_decl(const RzTypeDB *typedb, RZ_NONNULL const RzType *type, RZ_NONNULL const char *identifier, RZ_NONNULL struct HelperBufs *bufs) {
-	rz_return_val_if_fail(typedb && type && identifier && bufs, NULL);
-
-	RzStrBuf *buf = rz_strbuf_new("");
-	switch (type->kind) {
-	case RZ_TYPE_KIND_IDENTIFIER: {
-		rz_return_val_if_fail(type->identifier.name, NULL);
-		// Here it can be any of the RzBaseType
-		RzBaseType *btype = rz_type_db_get_base_type(typedb, type->identifier.name);
-		if (!btype) {
-			rz_strbuf_append(buf, "unknown_t");
-		} else {
-			// If the structure/union is anonymous, then we put declaration inline,
-			// if not - just the name
-			if (!strncmp(type->identifier.name, "anonymous ", 10)) {
-				char *btypestr = btype->kind == RZ_BASE_TYPE_KIND_TYPEDEF ? strdup(btype->name) : rz_type_db_base_type_as_string(typedb, btype);
-				if (type->identifier.is_const) {
-					rz_strbuf_appendf(buf, "const %s", btypestr);
-				} else {
-					rz_strbuf_append(buf, btypestr);
-				}
-				free(btypestr);
-			} else {
-				if (type->identifier.is_const) {
-					rz_strbuf_append(buf, "const ");
-				}
-				switch (btype->kind) {
-				case RZ_BASE_TYPE_KIND_UNION:
-					rz_strbuf_append(buf, "union ");
-					break;
-				case RZ_BASE_TYPE_KIND_STRUCT:
-					rz_strbuf_append(buf, "struct ");
-					break;
-				default:
-					break;
-				}
-				rz_strbuf_append(buf, btype->name);
-			}
-		}
-		rz_strbuf_appendf(buf, " %s%s%s", rz_strbuf_get(bufs->ptrbuf), identifier, rz_strbuf_get(bufs->arraybuf));
-		break;
-	}
-	case RZ_TYPE_KIND_POINTER: {
-		// A pointer to the function is a special case
-		if (rz_type_is_callable_ptr_nested(type)) {
-			char *typestr = rz_type_callable_ptr_as_string(typedb, type);
-			rz_strbuf_append(buf, typestr);
-			free(typestr);
-		} else {
-			if (type->pointer.is_const) {
-				rz_strbuf_prepend(bufs->ptrbuf, "* const ");
-			} else {
-				rz_strbuf_prepend(bufs->ptrbuf, "*");
-			}
-			char *typestr = type_as_string_identifier_decl(typedb, type->pointer.type, identifier, bufs);
-			rz_strbuf_append(buf, typestr);
-			free(typestr);
-		}
-		break;
-	}
-	case RZ_TYPE_KIND_ARRAY: {
-		rz_strbuf_appendf(bufs->arraybuf, "[%" PFMT64d "]", type->array.count);
-		char *typestr = type_as_string_identifier_decl(typedb, type->array.type, identifier, bufs);
-		rz_strbuf_append(buf, typestr);
-		free(typestr);
-		break;
-	}
-	case RZ_TYPE_KIND_CALLABLE: {
-		char *callstr = rz_type_callable_as_string(typedb, type->callable);
-		rz_strbuf_append(buf, callstr);
-		free(callstr);
-		break;
-	}
-	}
-	return rz_strbuf_drain(buf);
-}
-
 /**
  * \brief Returns the type C representation
  *
@@ -1018,11 +816,8 @@ static char *type_as_string_identifier_decl(const RzTypeDB *typedb, RZ_NONNULL c
  */
 RZ_API RZ_OWN char *rz_type_as_string(const RzTypeDB *typedb, RZ_NONNULL const RzType *type) {
 	rz_return_val_if_fail(typedb && type, NULL);
-	struct HelperBufs bufs;
-	helper_bufs_init(&bufs);
-	char *r = type_as_string(typedb, type, &bufs);
-	helper_bufs_fini(&bufs);
-	return r;
+
+	return rz_type_as_pretty_string(typedb, type, NULL, RZ_TYPE_PRINT_ZERO_VLA | RZ_TYPE_PRINT_NO_END_SEMICOLON | RZ_TYPE_PRINT_ANONYMOUS, 0);
 }
 
 /**
@@ -1033,11 +828,8 @@ RZ_API RZ_OWN char *rz_type_as_string(const RzTypeDB *typedb, RZ_NONNULL const R
  */
 RZ_API RZ_OWN char *rz_type_declaration_as_string(const RzTypeDB *typedb, RZ_NONNULL const RzType *type) {
 	rz_return_val_if_fail(typedb && type, NULL);
-	struct HelperBufs bufs;
-	helper_bufs_init(&bufs);
-	char *r = type_as_string_decl(typedb, type, &bufs);
-	helper_bufs_fini(&bufs);
-	return r;
+
+	return rz_type_as_pretty_string(typedb, type, NULL, RZ_TYPE_PRINT_ZERO_VLA | RZ_TYPE_PRINT_NO_END_SEMICOLON, 1); // one level unfold
 }
 
 /**
@@ -1048,11 +840,307 @@ RZ_API RZ_OWN char *rz_type_declaration_as_string(const RzTypeDB *typedb, RZ_NON
  */
 RZ_API RZ_OWN char *rz_type_identifier_declaration_as_string(const RzTypeDB *typedb, RZ_NONNULL const RzType *type, RZ_NONNULL const char *identifier) {
 	rz_return_val_if_fail(typedb && type, NULL);
-	struct HelperBufs bufs;
-	helper_bufs_init(&bufs);
-	char *r = type_as_string_identifier_decl(typedb, type, identifier, &bufs);
-	helper_bufs_fini(&bufs);
-	return r;
+
+	return rz_type_as_pretty_string(typedb, type, identifier, RZ_TYPE_PRINT_ZERO_VLA | RZ_TYPE_PRINT_NO_END_SEMICOLON | RZ_TYPE_PRINT_UNFOLD_ANON_ONLY_STRICT, 1); // one level unfold (for anonymous only)
+}
+
+struct PrettyHelperBufs {
+	RzStrBuf *typename;
+	RzStrBuf *pointerbuf;
+	RzStrBuf *arraybuf;
+};
+
+static bool type_decl_as_pretty_string(const RzTypeDB *typedb, const RzType *type, HtPP *used_types, struct PrettyHelperBufs phbuf, bool *self_ref, char **self_ref_typename, bool zero_vla, bool print_anon, bool show_typedefs) {
+	rz_return_val_if_fail(typedb && type && used_types && self_ref, false);
+
+	bool is_anon = false;
+	switch (type->kind) {
+	case RZ_TYPE_KIND_IDENTIFIER: {
+		if (!type->identifier.name) {
+			return false;
+		}
+		is_anon = !strncmp(type->identifier.name, "anonymous ", strlen("anonymous "));
+		*self_ref = false;
+		ht_pp_find(used_types, type->identifier.name, self_ref);
+		*self_ref = *self_ref && !is_anon; // no self_ref for anon types
+		*self_ref_typename = *self_ref ? strdup(type->identifier.name) : NULL;
+
+		RzBaseType *btype = rz_type_db_get_base_type(typedb, type->identifier.name);
+		if (!btype) {
+			rz_strbuf_append(phbuf.typename, "unknown_t");
+		} else {
+			switch (btype->kind) {
+			case RZ_BASE_TYPE_KIND_STRUCT:
+				rz_strbuf_appendf(phbuf.typename, "%sstruct", type->identifier.is_const ? "const " : "");
+				if (!is_anon || print_anon) {
+					rz_strbuf_appendf(phbuf.typename, " %s", btype->name);
+				}
+				break;
+			case RZ_BASE_TYPE_KIND_UNION:
+				rz_strbuf_appendf(phbuf.typename, "%sunion", type->identifier.is_const ? "const " : "");
+				if (!is_anon || print_anon) {
+					rz_strbuf_appendf(phbuf.typename, " %s", btype->name);
+				}
+				break;
+			case RZ_BASE_TYPE_KIND_ENUM:
+				rz_strbuf_appendf(phbuf.typename, "%senum", type->identifier.is_const ? "const " : "");
+				if (!is_anon || print_anon) {
+					rz_strbuf_appendf(phbuf.typename, " %s", btype->name);
+				}
+				break;
+			case RZ_BASE_TYPE_KIND_TYPEDEF: {
+				if (show_typedefs) {
+					char *typestr = rz_type_as_string(typedb, btype->type);
+					if (!typestr) {
+						RZ_LOG_ERROR("Failed to get type representation of typedef of base type: %s\n", btype->name);
+						return false;
+					}
+					rz_strbuf_appendf(phbuf.typename, "typedef %s", typestr);
+					free(typestr);
+				} else {
+					rz_strbuf_append(phbuf.typename, btype->name);
+				}
+				break;
+			}
+			case RZ_BASE_TYPE_KIND_ATOMIC:
+				rz_strbuf_appendf(phbuf.typename, "%s%s", type->identifier.is_const ? "const " : "", btype->name);
+				break;
+			default:
+				rz_warn_if_reached();
+				break;
+			}
+		}
+		break;
+	}
+	case RZ_TYPE_KIND_POINTER:
+		if (rz_type_is_callable_ptr_nested(type)) { // function pointers
+			char *typestr = rz_type_callable_ptr_as_string(typedb, type);
+			rz_strbuf_append(phbuf.typename, typestr);
+			free(typestr);
+		} else {
+			type_decl_as_pretty_string(typedb, type->pointer.type, used_types, phbuf, self_ref, self_ref_typename, zero_vla, print_anon, show_typedefs);
+			rz_strbuf_append(phbuf.pointerbuf, "*");
+			rz_strbuf_appendf(phbuf.pointerbuf, "%s", type->pointer.is_const ? " const " : "");
+		}
+		break;
+	case RZ_TYPE_KIND_ARRAY:
+		if (type->array.count) {
+			rz_strbuf_appendf(phbuf.arraybuf, "[%" PFMT64d "]", type->array.count);
+		} else { // variable length arrays
+			rz_strbuf_appendf(phbuf.arraybuf, "[%s]", zero_vla ? "0" : "");
+		}
+		type_decl_as_pretty_string(typedb, type->array.type, used_types, phbuf, self_ref, self_ref_typename, zero_vla, print_anon, show_typedefs);
+		break;
+	case RZ_TYPE_KIND_CALLABLE: {
+		char *callstr = rz_type_callable_as_string(typedb, type->callable);
+		rz_strbuf_append(phbuf.typename, callstr);
+		free(callstr);
+		break;
+	}
+	default:
+		rz_warn_if_reached();
+		break;
+	}
+
+	return true;
+}
+
+static char *type_as_pretty_string(const RzTypeDB *typedb, const RzType *type, const char *identifier, HtPP *used_types, unsigned int opts, int unfold_level, int indent_level) {
+	rz_return_val_if_fail(typedb && type, NULL);
+
+	if (unfold_level < 0) { // recursion base case
+		return NULL;
+	}
+	bool multiline = opts & RZ_TYPE_PRINT_MULTILINE;
+	bool anon_only = opts & RZ_TYPE_PRINT_UNFOLD_ANON_ONLY;
+	bool anon_only_strict = opts & RZ_TYPE_PRINT_UNFOLD_ANON_ONLY_STRICT;
+	bool zero_vla = opts & RZ_TYPE_PRINT_ZERO_VLA;
+	bool print_anon = opts & RZ_TYPE_PRINT_ANONYMOUS;
+	bool no_end_semicolon = opts & RZ_TYPE_PRINT_NO_END_SEMICOLON;
+	no_end_semicolon = no_end_semicolon && (indent_level == 0); // indent_level needs to be zero for the last semicolon
+	bool end_newline = opts & RZ_TYPE_PRINT_END_NEWLINE;
+	end_newline = end_newline && (indent_level == 0); // only append newline for the outer type
+	bool show_typedefs = opts & RZ_TYPE_PRINT_SHOW_TYPEDEF;
+	if (indent_level == 0) { // for the root type, disregard anon_only
+		anon_only = false;
+	}
+	anon_only = anon_only || anon_only_strict;
+	bool unfold_all = !anon_only && unfold_level;
+	bool unfold_anon = unfold_level;
+	int indent = 0;
+	char *separator = " ";
+	if (multiline) {
+		indent = indent_level; // indent only if multiline
+		separator = "\n";
+	}
+
+	RzStrBuf *buf = rz_strbuf_new("");
+	for (int i = 0; i < indent; i++) {
+		rz_strbuf_append(buf, "\t");
+	}
+	RzStrBuf *typename = rz_strbuf_new("");
+	RzStrBuf *pointer_buf = rz_strbuf_new("");
+	RzStrBuf *array_buf = rz_strbuf_new("");
+	struct PrettyHelperBufs phbuf = { typename, pointer_buf, array_buf };
+	bool self_ref = false;
+	char *self_ref_typename = NULL;
+	bool decl = type_decl_as_pretty_string(typedb, type, used_types, phbuf, &self_ref, &self_ref_typename, zero_vla, print_anon, show_typedefs);
+	if (!decl) {
+		rz_strbuf_free(buf);
+		rz_strbuf_free(typename);
+		rz_strbuf_free(pointer_buf);
+		rz_strbuf_free(array_buf);
+		return NULL;
+	}
+	if (self_ref) { // in case of self referntial type
+		unfold_level = 0; // no unfold
+		unfold_anon = unfold_all = false;
+	} else if (self_ref_typename) {
+		ht_pp_insert(used_types, self_ref_typename, NULL); // add the type to the ht
+	}
+	RzBaseType *btype = NULL;
+	bool is_anon = false;
+	if (type->kind == RZ_TYPE_KIND_IDENTIFIER) {
+		is_anon = !strncmp(type->identifier.name, "anonymous ", 10);
+		btype = rz_type_db_get_base_type(typedb, type->identifier.name);
+	} else if ((type->kind == RZ_TYPE_KIND_POINTER && rz_type_is_callable_ptr_nested(type)) || type->kind == RZ_TYPE_KIND_CALLABLE) {
+		identifier = NULL; // no need to separately print identifier for function pointers or functions
+	}
+	char *typename_str = rz_strbuf_drain(phbuf.typename);
+	char *pointer_str = rz_strbuf_drain(phbuf.pointerbuf);
+	char *array_str = rz_strbuf_drain(phbuf.arraybuf);
+	rz_strbuf_append(buf, typename_str);
+
+	if (btype) {
+		bool not_empty; // to check if no members are present
+		switch (btype->kind) {
+		case RZ_BASE_TYPE_KIND_STRUCT:
+			if (unfold_all || (is_anon && unfold_anon)) {
+				rz_strbuf_append(buf, " {");
+				RzTypeStructMember *memb;
+				not_empty = rz_vector_len(&btype->struct_data.members);
+				if (not_empty) {
+					rz_strbuf_appendf(buf, "%s", multiline ? "\n" : " ");
+				}
+				rz_vector_foreach(&btype->struct_data.members, memb) {
+					char *unfold = type_as_pretty_string(typedb, memb->type, memb->name, used_types, opts, unfold_level - 1, indent_level + 1);
+					rz_strbuf_appendf(buf, "%s%s", unfold, separator);
+					free(unfold);
+				}
+				for (int i = 0; i < indent; i++) {
+					rz_strbuf_append(buf, "\t");
+				}
+				rz_strbuf_append(buf, "}");
+			}
+			break;
+		case RZ_BASE_TYPE_KIND_UNION:
+			if (unfold_all || (is_anon && unfold_anon)) {
+				rz_strbuf_append(buf, " {");
+				RzTypeUnionMember *memb;
+				not_empty = rz_vector_len(&btype->union_data.members);
+				if (not_empty) {
+					rz_strbuf_appendf(buf, "%s", multiline ? "\n" : " ");
+				}
+				rz_vector_foreach(&btype->union_data.members, memb) {
+					char *unfold = type_as_pretty_string(typedb, memb->type, memb->name, used_types, opts, unfold_level - 1, indent_level + 1);
+					rz_strbuf_appendf(buf, "%s%s", unfold, separator);
+					free(unfold);
+				}
+				for (int i = 0; i < indent; i++) {
+					rz_strbuf_append(buf, "\t");
+				}
+				rz_strbuf_append(buf, "}");
+			}
+			break;
+		case RZ_BASE_TYPE_KIND_ENUM:
+			if (unfold_all || (is_anon && unfold_anon)) {
+				RzTypeEnumCase *cas;
+				rz_strbuf_append(buf, " {");
+				if (multiline) {
+					indent++; // no recursive call, so manually need to update indent
+				}
+				not_empty = rz_vector_len(&btype->enum_data.cases);
+				if (not_empty) {
+					rz_strbuf_appendf(buf, "%s", multiline ? "\n" : " ");
+				}
+				rz_vector_foreach(&btype->enum_data.cases, cas) {
+					for (int i = 0; i < indent; i++) {
+						rz_strbuf_append(buf, "\t");
+					}
+					rz_strbuf_appendf(buf, "%s = 0x%" PFMT64x ",%s", cas->name, cas->val, separator);
+				}
+				if (not_empty) {
+					rz_strbuf_slice(buf, 0, rz_strbuf_length(buf) - 2);
+					rz_strbuf_append(buf, separator);
+				}
+				if (multiline) {
+					indent--; // restore the original value
+				}
+				for (int i = 0; i < indent; i++) {
+					rz_strbuf_append(buf, "\t");
+				}
+				rz_strbuf_append(buf, "}");
+			}
+			break;
+		case RZ_BASE_TYPE_KIND_TYPEDEF:
+			if (show_typedefs && !rz_type_is_callable_ptr_nested(btype->type)) { // if not a callable
+				rz_strbuf_appendf(buf, " %s", btype->name);
+			}
+			break;
+		case RZ_BASE_TYPE_KIND_ATOMIC:
+			break;
+		default:
+			rz_warn_if_reached();
+			break;
+		}
+	}
+
+	if (rz_str_nlen(pointer_str, 1) != 0 || identifier || rz_str_nlen(array_str, 1) != 0) {
+		rz_strbuf_append(buf, " "); // add space only if the type is pointer or an array or has an identifier
+	}
+	rz_strbuf_appendf(buf, "%s%s%s", pointer_str ? pointer_str : "", identifier ? identifier : "", array_str ? array_str : "");
+	if (!no_end_semicolon) {
+		rz_strbuf_append(buf, ";");
+	}
+	if (end_newline) {
+		rz_strbuf_append(buf, "\n");
+	}
+	if (self_ref_typename) {
+		ht_pp_delete(used_types, self_ref_typename);
+		free(self_ref_typename);
+	}
+	free(typename_str);
+	free(pointer_str);
+	free(array_str);
+
+	char *pretty_type = rz_strbuf_drain(buf);
+	return pretty_type;
+}
+
+/**
+ * \brief Return a string contining the type pretty printed according to the options provided
+ *
+ * \param typedb typedb for the current analysis
+ * \param type type to be pretty printed
+ * \param identifier name of the variable of the given type (RZ_NULLABLE)
+ * \param opts options for pretty printing (see RzTypePrintOpts)
+ * \param unfold_level level of unfolding to do in case of nested structures/unions (any negative number means maximum unfolding, i.e. INT32_MAX. 0 means no unfolding, just the typename and identifier, if any)
+ * \return char* string in pretty printed form
+ */
+RZ_API RZ_OWN char *rz_type_as_pretty_string(const RzTypeDB *typedb, RZ_NONNULL const RzType *type, RZ_NULLABLE const char *identifier, unsigned int opts, int unfold_level) {
+	rz_return_val_if_fail(typedb && type, NULL);
+
+	if (unfold_level < 0) { // any negative number means maximum unfolding
+		unfold_level = INT32_MAX;
+	}
+	HtPP *used_types = ht_pp_new0(); // use a hash table to keep track of unfolded types
+	if (!used_types) {
+		RZ_LOG_ERROR("Failed to create hashtable while pretty printing types")
+		return NULL;
+	}
+	char *pretty_type = type_as_pretty_string(typedb, type, identifier, used_types, opts, unfold_level, 0);
+	ht_pp_free(used_types);
+	return pretty_type;
 }
 
 /**
